/***********************************************************************
 *
 Copyright  1995 - 1998, 3Com Corporation or its subsidiaries ("3Com").  
 All rights reserved.
   
 This software may be copied and used solely for developing products for 
 the Palm Computing platform and for archival and backup purposes.  Except 
 for the foregoing, no part of this software may be reproduced or transmitted 
 in any form or by any means or used to make any derivative work (such as 
 translation, transformation or adaptation) without express written consent 
 from 3Com.

 3Com reserves the right to revise this software and to make changes in content 
 from time to time without obligation on the part of 3Com to provide notification 
 of such revision or changes.  
 3COM MAKES NO REPRESENTATIONS OR WARRANTIES THAT THE SOFTWARE IS FREE OF ERRORS 
 OR THAT THE SOFTWARE IS SUITABLE FOR YOUR USE.  THE SOFTWARE IS PROVIDED ON AN 
 "AS IS" BASIS.  3COM MAKES NO WARRANTIES, TERMS OR CONDITIONS, EXPRESS OR IMPLIED, 
 EITHER IN FACT OR BY OPERATION OF LAW, STATUTORY OR OTHERWISE, INCLUDING WARRANTIES, 
 TERMS, OR CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND 
 SATISFACTORY QUALITY.

 TO THE FULL EXTENT ALLOWED BY LAW, 3COM ALSO EXCLUDES FOR ITSELF AND ITS SUPPLIERS 
 ANY LIABILITY, WHETHER BASED IN CONTRACT OR TORT (INCLUDING NEGLIGENCE), FOR 
 DIRECT, INCIDENTAL, CONSEQUENTIAL, INDIRECT, SPECIAL, OR PUNITIVE DAMAGES OF 
 ANY KIND, OR FOR LOSS OF REVENUE OR PROFITS, LOSS OF BUSINESS, LOSS OF INFORMATION 
 OR DATA, OR OTHER FINANCIAL LOSS ARISING OUT OF OR IN CONNECTION WITH THIS SOFTWARE, 
 EVEN IF 3COM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

 3Com, HotSync, Palm Computing, and Graffiti are registered trademarks, and 
 Palm III and Palm OS are trademarks of 3Com Corporation or its subsidiaries.

 IF THIS SOFTWARE IS PROVIDED ON A COMPACT DISK, THE OTHER SOFTWARE AND 
 DOCUMENTATION ON THE COMPACT DISK ARE SUBJECT TO THE LICENSE AGREEMENT 
 ACCOMPANYING THE COMPACT DISK.

 *************************************************************************
 *
 * PROJECT:  Pilot 2.0
 *
 * FILE:     MailDB.h
 *
 * AUTHOR:	 Art Lamb: May 30, 1996
 *
 * DECLARER: Mail
 *
 * DESCRIPTION:
 *	  This file defines the database structures and functions of the Mail
 *   application.
 *
 ***********************************************************************/

/************************************************************
 *
 * Mail database constants.
 *
 *************************************************************/

// String resource used to initialize application info block.
#define mailLocalizedAppInfoStr	1000

// Dirty flags for to do application info
#define mailSortByDirty	0x0001


// Sort orders
#define sortByDate					0
#define sortByFrom					1
#define sortBySubject				2


#define noUnsendMessage				-1

/************************************************************
 *
 * Mail database structures.
 *
 *************************************************************/

// The format of a packed mail record is as follows:
//
//	DateType			 		date;
//	TimeType			 		time;
//	MailDBRecordFlags		flags;
//	char []					subject;			// null terminated
//	char []					from;				// null terminated
//	char []					to;				// null terminated
//	char []					cc;				// null terminated
//	char []					bcc;				// null terminated
//	char []					replyTo;			// null terminated
// char []					sentTo;			// null terminated
//	char []					body;				// null terminated


// Application info block 
typedef struct {
   UInt				renamedCategories;	// bitfield of categories with a different name
	char 				categoryLabels [dmRecNumCategories] [dmCategoryLength];
	Byte 				categoryUniqIDs [dmRecNumCategories];
	Byte				lastUniqID;			// Uniq IDs generated by the device are between
												// 0 - 127.  Those from the PC are 128 - 255.
	Byte				reserved1;			// from the compiler word aligning things
	Word				reserved2;
	UInt				dirtyAppInfo;
	Byte				sortOrder;
	ULong				unsentMessage;		// unique id of unsend message.
	Word				signatureOffset;
} MailAppInfoType;

typedef MailAppInfoType * MailAppInfoPtr;

typedef enum { priorityHigh, priorityNormal, priorityLow } MailPriorityType;

typedef enum { sentTo, sentCC, sentBCC } MailAddressingType;

typedef struct {
	Word				read					: 1;
	Word				signature			: 1;
	Word				confirmRead			: 1;
	Word				confirmDelivery	: 1;
	Word				priority				: 2;		// MailPriorityType
	Word				addressing			: 2;		// MailAddressingType
	Word				reserved				: 8;
} MailFlagsType;


// The following structure doesn't really exist.  The first field
// varies depending on the data present.  However, it is convient
// (and less error prone) to use when accessing the other information.
typedef struct {
	DateType			 		date;
	TimeType					time;
	MailFlagsType			flags;
	char						firstField;
} MailPackedDBRecordType;

typedef MailPackedDBRecordType * MailPackedDBRecordPtr;


// MailItemType
//
// This is the format of a to do record used by the application.  All 
// pointers are either NULL or point to data within the PackedDB record.
// All strings are null character terminated.
typedef struct {
	DateType 				date;
	TimeType					time;
	MailFlagsType			flags;
	CharPtr					subject;
	CharPtr					from;
	CharPtr					to;
	CharPtr					cc;
	CharPtr					bcc;
	CharPtr					replyTo;
	CharPtr					sentTo;
	CharPtr					body;
} MailDBRecordType;

typedef MailDBRecordType * MailDBRecordPtr;



//This structure should match the order of the fields in the record.
typedef enum { 
	mailFieldDate,
	mailFieldTime,
	mailFieldFlags,
	mailFieldSubject,
	mailFieldFrom,
	mailFieldTo,
	mailFieldCC,
	mailFieldBCC,
	mailFieldReplyTo,
	mailFieldSentTo,
	mailFieldBody } MailRecordFieldType;


// Used by MailChangeRecord function.
typedef struct {
	unsigned 	date		:1;
	unsigned 	time		:1;
	unsigned		flags		:1;
	unsigned		subject	:1;
	unsigned		from		:1;
	unsigned		to			:1;
	unsigned		cc			:1;
	unsigned		bcc		:1;
	unsigned		replyTo	:1;
	unsigned		sentTo	:1;
	unsigned		body		:1;
} MailChangedFieldsType;




/************************************************************
 *
 * Mail database routines.
 *
 *************************************************************/
#ifdef __cplusplus
extern "C" {
#endif


Err		MailAppInfoInit (DmOpenRef dbP);

VoidHand MailGetAppInfo (DmOpenRef dbP);

Byte 		MailGetSortOrder (DmOpenRef dbP);

Err		MailChangeSortOrder (DmOpenRef dbP, Byte sortOrder);

void 		MailSort (DmOpenRef dbP);

ULong		MailGetUnsentMessage (DmOpenRef dbP);

void		MailSetUnsentMessage (DmOpenRef dbP, ULong uniqueID);

VoidHand MailGetSignature (DmOpenRef dbP);

void 		MailSetSignature (DmOpenRef dbP, VoidHand h);



void 		MailParseAddress (CharPtr address, Boolean realName, CharPtr buffer,
				Word bufferLen);
				
Err 		MailNewRecord (DmOpenRef dbP, MailDBRecordPtr r, UInt *indexP);

Err		MailAttachRecord (DmOpenRef dbP, VoidHand recordH, WordPtr indexP);

Err 		MailGetRecord (DmOpenRef dbP, Word index, MailDBRecordPtr r,
				VoidHand * handleP);

Err 		MailChangeRecord (DmOpenRef dbP, WordPtr index, 
				MailDBRecordPtr r, MailChangedFieldsType changedFields);

void		MailUnpackRecord (MailPackedDBRecordPtr src, MailDBRecordPtr dest);

Err 		MailChangeRecordField (DmOpenRef dbP, WordPtr indexP, 
				MailRecordFieldType recordField, VoidPtr data);

Err 		MailInsertIntoField (DmOpenRef dbP, Word index, 
				MailRecordFieldType recordField, CharPtr insertData, 
				Word insertLen, Word insertOffset);

Err 		MailSortRecord (DmOpenRef dbP, WordPtr indexP);


void 		ECMailDBValidate (DmOpenRef dbP);


#ifdef __cplusplus
}
#endif


