/*******************************************************************
 Copyright  1995 - 1998, 3Com Corporation or its subsidiaries ("3Com").  
 All rights reserved.
   
 This software may be copied and used solely for developing products for 
 the Palm Computing platform and for archival and backup purposes.  Except 
 for the foregoing, no part of this software may be reproduced or transmitted 
 in any form or by any means or used to make any derivative work (such as 
 translation, transformation or adaptation) without express written consent 
 from 3Com.

 3Com reserves the right to revise this software and to make changes in content 
 from time to time without obligation on the part of 3Com to provide notification 
 of such revision or changes.  
 3COM MAKES NO REPRESENTATIONS OR WARRANTIES THAT THE SOFTWARE IS FREE OF ERRORS 
 OR THAT THE SOFTWARE IS SUITABLE FOR YOUR USE.  THE SOFTWARE IS PROVIDED ON AN 
 "AS IS" BASIS.  3COM MAKES NO WARRANTIES, TERMS OR CONDITIONS, EXPRESS OR IMPLIED, 
 EITHER IN FACT OR BY OPERATION OF LAW, STATUTORY OR OTHERWISE, INCLUDING WARRANTIES, 
 TERMS, OR CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND 
 SATISFACTORY QUALITY.

 TO THE FULL EXTENT ALLOWED BY LAW, 3COM ALSO EXCLUDES FOR ITSELF AND ITS SUPPLIERS 
 ANY LIABILITY, WHETHER BASED IN CONTRACT OR TORT (INCLUDING NEGLIGENCE), FOR 
 DIRECT, INCIDENTAL, CONSEQUENTIAL, INDIRECT, SPECIAL, OR PUNITIVE DAMAGES OF 
 ANY KIND, OR FOR LOSS OF REVENUE OR PROFITS, LOSS OF BUSINESS, LOSS OF INFORMATION 
 OR DATA, OR OTHER FINANCIAL LOSS ARISING OUT OF OR IN CONNECTION WITH THIS SOFTWARE, 
 EVEN IF 3COM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

 3Com, HotSync, Palm Computing, and Graffiti are registered trademarks, and 
 Palm III and Palm OS are trademarks of 3Com Corporation or its subsidiaries.

 IF THIS SOFTWARE IS PROVIDED ON A COMPACT DISK, THE OTHER SOFTWARE AND 
 DOCUMENTATION ON THE COMPACT DISK ARE SUBJECT TO THE LICENSE AGREEMENT 
 ACCOMPANYING THE COMPACT DISK.

 *****************************************************************************
 * FileName:
 *		ToDoDB.h
 *
 * Description:
 *		Header for the To Do Manager
 *
 * History:
 *   	1/19/95  rsf - Created
 *
 *******************************************************************/

#ifndef __TDTODOMGR_H__
#define __TDTODOMGR_H__

#include <DateTime.h>

#define LocalizedAppInfoStr			1000

#define todoLabelLength		12
#define todoNumFields		16

#define toDoMaxPriority		5

// Dirty flags for to do application info
#define toDoSortByPriorityDirty	0x0001

// Sort orders
#define soDueDatePriority		0
#define soPriorityDueDate		1
#define soCategoryPriority		2
#define soCategoryDueDate		3


typedef struct {
    UInt				renamedCategories;	// bitfield of categories with a different name
	char 				categoryLabels[dmRecNumCategories][dmCategoryLength];
	Byte 				categoryUniqIDs[dmRecNumCategories];
	Byte				lastUniqID;	// Uniq IDs generated by the device are between
										// 0 - 127.  Those from the PC are 128 - 255.
	Byte				reserved1;	// from the compiler word aligning things
	Word				reserved2;
	UInt				dirtyAppInfo;
	Byte				sortOrder;
} ToDoAppInfoType;

typedef ToDoAppInfoType * ToDoAppInfoPtr;

// ToDoDBRecord.
//
// All strings are null terminated and appear within the record.
//
typedef struct {
	DateType dueDate;
	unsigned char priority;		// high bit is complete flag
	char description;
} ToDoDBRecord;

typedef ToDoDBRecord * ToDoDBRecordPtr;

// sizeDBRecord is the minimum size for the ToDoDBRecord structure.
// It assumes empty 'description' and 'note' fields.
// sizeDBRecord allocates an extra byte for the null character
// for the note string.
#define sizeDBRecord (sizeof (ToDoDBRecord) + 1)

// ToDoItemType
//
// This is the format of a to do record used by the application.  All 
// pointers are either NULL or point to data within the PackedDB record.
// All strings are null character terminated.
//
typedef struct {
	DateType 			dueDate;
	unsigned char 		priority;		// high bit is complete flag
	CharPtr				description;
	CharPtr				note;
} ToDoItemType;

typedef ToDoItemType * ToDoItemPtr;


#define completeFlag 	0x80
#define priorityOnly 	~completeFlag
#define toDoNoDueDate	0xffff


// Used for ToDoChangeRecord.
typedef enum  { 
	toDoPriority,
	toDoComplete,
	toDoDueDate,
	toDoDescription,
	toDoNote,
	toDoCategory } ToDoRecordFieldType;



#ifdef __cplusplus
extern "C" {
#endif


//-------------------------------------------------------------------
// Routines
//-------------------------------------------------------------------
Err ToDoNewRecord(DmOpenRef dbP, ToDoItemPtr item, UInt category, UInt *index);

Err ToDoInsertNewRecord (DmOpenRef dbP, UIntPtr index);

Err ToDoChangeRecord(DmOpenRef dbP, UIntPtr index, 
	ToDoRecordFieldType changedField, VoidPtr data);

Err ToDoChangeSortOrder(DmOpenRef dbP, Byte sortOrder);

Err ToDoAppInfoInit(DmOpenRef dbP);

VoidHand ToDoGetAppInfo  (DmOpenRef dbP);

Byte ToDoGetSortOrder (DmOpenRef dbP);

void ToDoSort (DmOpenRef dbP);


#ifdef __cplusplus
}
#endif


#endif
