
        Appl    "Lotto", 'Loto'                         ; Define name and creator ID.Creator ID is sometimes called AppType

        include "Pilot.inc"                             ; structs, constants
        include "Startup.inc"                           ; common startup code
; Application-defined resource ids

kidbAIB                 equ     $7FFE
LottoCreator            equ     $4C6F746F               ; 'Loto'
LottoVersion            equ     $0100

        include "Lotto.inc"


TMin    equ     2
TMax    equ     64
NMin    equ     1
NMax    equ     09
INIT    equ     0

; Several registers have special purposes.
; APIs preserve all registers except D0, D1, A0, and A1. APIs that return pointers
; return them in the A0 register. All other APIs return their values in the D0 register.
; A7 is the stack pointer.
; A6 is used as a local frame base pointer.
; A5 points to a location within the application's data section. All global
; variable accesses are performed relative to A5.

; Some global variables (a5-relative)

        data

global  TStr1.64                                        ; Buffer
global  TStr2.64

Total   dc.w    45
Number  dc.w    06
Drawn   dc.b    0
Init    dc.b    0

        code


proc PilotMain(cmd.w, cmdPBP.l, launchFlags.w)
local   err.w
local   evt.EventType
beginproc
        tst.w   cmd(a6)                                 ; sysAppLaunchCmdNormalLaunch is 0
        bne     PmReturn                                ; Not a normal launch, bag out
        systrap FrmGotoForm(#kidfMain.w)

PmEventLoop                                             ; Doze until an event arrives
        systrap EvtGetEvent(&evt(a6), #evtWaitForever.w)
        systrap SysHandleEvent(&evt(a6))                ; First handler is system
        tst.b   d0                                      ; handled?
        bne.s   PmEventDone                             ; yep
        systrap MenuHandleEvent(&0, &evt(a6), &err(a6))
        tst.b   d0                                      ; handled?
        bne.s   PmEventDone                             ; yep
        call    ApplicationHandleEvent(&evt(a6))        ; Third is application handler
        tst.b   d0                                      ; handled?
        bne.s   PmEventDone                             ; yep
        systrap FrmDispatchEvent(&evt(a6))
                                                        ; Return from PilotMain when an
PmEventDone                                             ; appStopEvent is received
        cmpi.w  #appStopEvent,evt+EventType.eType(a6)   ; time to stop?
        bne.s   PmEventLoop                             ; nope, loop until it is
PmReturn
        moveq   #0, d0
endproc ; PilotMain()


proc ApplicationHandleEvent(pevt.l)
beginproc
        move.l  a3,-(a7)                                ; Save registers we're going to trash
        movea.l pevt(a6),a0

        cmpi.w  #frmLoadEvent,EventType.eType(a0)
        beq     FormLoadHandler
        cmpi.w  #appStopEvent,EventType.eType(a0)
        beq     ApplicationStopHandler
        bra     ApplicationEventNotHandled

FormLoadHandler                                         ; Initialize the form and make it active
        systrap FrmInitForm(EventType.data+frmLoad.formID(a0).w)
        move.l  a0, a3
        systrap FrmSetActiveForm(a0.l)
        systrap FrmSetEventHandler(a3.l, &MainFormHandleEvent(pc))

        systrap TimGetSeconds()                         ; New seed in D0.L
        systrap SysRandom(d0.l)                         ; Returns a word value in D0.W
        move.b  #0,Drawn(a5)

        move.w  #1,d0
        move.b  d0,-(a7)
        trap    #15
        dc.w    sysTrapFntSetFont
        addq.l  #2,a7

        systrap FrmGetActiveForm()                      ; Get form pointer a0
        systrap FrmDrawForm(a0.l)                       ; Draw the form
        call    Rectangle(#47.w,#28.w,#17.w,#15.w,#1.w)
        call    Rectangle(#91.w,#56.w,#17.w,#15.w,#1.w)

        systrap PrefGetAppPreferences(#LottoCreator.l,#LottoVersion.w,&Total(a5),#8.w)
        tst.b   d0
        bne     Okay
        move.w  #45,Total(a5)
        move.w  #6,Number(a5)

Okay    move.w  #INIT,d0
        jsr     ChangeFirst(pc)
        move.w  #INIT,d0
        jsr     ChangeScnd(pc)

        bra     ApplicationEventNotHandled

ApplicationStopHandler                                  ; Application Stop Handler
        systrap PrefSetAppPreferences(#LottoCreator.l,#LottoVersion.w,&Total(a5),#8.w)
        systrap FrmSaveAllForms();
        systrap FrmCloseAllForms();

ApplicationEventNotHandled
        clr.b   d0

ApplicationEventReturn
        move.l  (a7)+,a3
endproc ; ApplicationHandleEvent()



proc MainFormHandleEvent(pevt.l)
beginproc
        movem.l a2,-(a7)                                ; Save registers we're going to trash
        movea.l pevt(a6),a0                             ; a0 = pevt
        move.w  EventType.eType(a0), d0                 ; d0 = event type
        cmp.w   #keyDownEvent, d0                       ; Handle key down Events
        beq     KeyboardHandler
        cmp.w   #menuEvent,d0                           ; Handle Menu Events
        beq     MenuHandler
        cmp.w   #ctlSelectEvent,d0                      ; Handle Control Events
        beq     ControlHandler
        cmp.w   #ctlRepeatEvent,d0                      ; Handle Control Events
        beq     RepeatHandler
        clr.w   d0
        bra     MFHReturn

ControlHandler
        move.w  EventType.data+ctlEnter.controlID(a0),d0
        cmp.w   #kidcMake,d0
        beq     PageUpDownMakeHandler
        bra     MFHNotHandled

RepeatHandler
        move.w  EventType.data+ctlRepeat.controlID(a0),d0
        cmp.w   #kidrUp1,d0
        beq     Change1
        cmp.w   #kidrDown1,d0
        beq     Change1
        cmp.w   #kidrUp2,d0
        beq     Change2
        cmp.w   #kidrDown2,d0
        beq     Change2
        bra     MFHNotHandled

MenuHandler
        move.w  EventType.data+menu.itemID(a0), d0
        cmp.w   #kidmCopy,d0
        beq     MenuCopyHandler
        cmp.w   #kidmAbout,d0
        bne     MFHHandled
        systrap FrmAlert(#kidrAboutAlert.w)
        bra     MFHHandled

MenuCopyHandler
        lea.l   TStr2(a5),a2
        systrap StrLen(a2)                              ; Length of string in D0.W
        systrap ClipboardAddItem(#clipboardText.w,a2.l,d0.w)
        bra     MFHHandled

KeyboardHandler
        move.w  EventType.data+keyDown.chr(a0), d0
        cmp.w   #pageDownChr, d0
        beq     PageUpDownMakeHandler
        cmp.w   #pageUpChr, d0
        beq     PageUpDownMakeHandler
        bra     MFHNotHandled

Change1
        jsr     ChangeFirst(pc)
        bra     MFHNotHandled

Change2
        jsr     ChangeScnd(pc)
        bra     MFHNotHandled

PageUpDownMakeHandler
        jsr     Make(pc)

MFHNotHandled
        clr.w   d0
        bra     MFHReturn

MFHHandled
        moveq.l #1, d0
MFHReturn
        movem.l (a7)+,a2
endproc ; MainFormHandleEvent()


proc ChangeFirst()
beginproc
        movem.l d2/a2,-(a7)                             ; Saves registers
        move.w  Number(a5),d2                           ; We want to incr/decr it
        cmpi.w  #INIT,d0
        beq.s   C1_Cmn
        cmpi.w  #kidrDown1,d0
        beq.s   C1_Down
        cmpi.w  #NMax,d2                                ; Can we increment it?
        beq     C1_Exit                                 ; No
        addi.w  #1,d2
        bra     C1_Cmn
C1_Down cmpi.w  #NMin,d2                                ; Can we decrement it?
        beq     C1_Exit
        subi.w  #1,d2
C1_Cmn  move.w  d2,Number(a5)
        move.w  d2,d0
        lea.l   TStr2(a5),a2                            ; Let A2 points to the Ascii-buffer
        cmpi.b  #9,d0                                   ; Is number > 9 ?
        bgt.s   C1_NZro                                 ; No! Insert a Zero
        move.b  #'0',(a2)+                              ; Insert zero
C1_NZro
        systrap StrIToA(a2.l,d0.l)
        lea.l   TStr2(a5),a2                            ; Let A2 points to the Ascii-buffer
        systrap WinDrawChars(a2.l,#2.w,#50.w,#30.w)
        call    Rectangle(#0.w,#93.w,#160.w,#18.w,#0.w)
;        lea.l   String(a5),a2
;        systrap WinDrawChars(a2.l,#2.w,#50.w,#30.w)
        move.b  #0,Drawn(a5)
C1_Exit
        movem.l  (a7)+,d2/a2                            ; Retrieve registers
endproc ; ChangeFirst()



proc ChangeScnd()
beginproc
        movem.l d2/a2,-(a7)                             ; Saves registers
        move.w  Total(a5),d2                            ; We want to incr/decr it
        cmpi.w  #INIT,d0
        beq.s   C2_Cmn
        cmpi.w  #kidrDown2,d0
        beq.s   C2_Down
        cmpi.w  #TMax,d2                                ; Can we increment it?
        beq     C2_Exit                                 ; No
        addi.w  #1,d2
        bra     C2_Cmn
C2_Down cmp.w   Number(a5),d2                           ; Can we decrement it?
        beq     C2_Exit
        subi.w  #1,d2
C2_Cmn  move.w  d2,Total(a5)
        move.w  d2,d0
        lea.l   TStr2(a5),a2                            ; Let A2 points to the Ascii-buffer
        cmpi.b  #9,d0                                   ; Is Total > 9 ?
        bgt.s   C2_NZro                                 ; No! Insert a Zero
        move.b  #'0',(a2)+                              ; Insert zero
C2_NZro
        systrap StrIToA(a2.l,d0.l)
        lea.l   TStr2(a5),a2                            ; Let A2 points to the Ascii-buffer
        systrap WinDrawChars(a2.l,#2.w,#94.w,#58.w)
        call    Rectangle(#0.w,#93.w,#160.w,#18.w,#0.w)
        move.b  #0,Drawn(a5)
C2_Exit
        movem.l  (a7)+,d2/a2                            ; Retrieve registers
endproc ; ChangeScnd()


proc Make()
local StrBuf.12
local Coord.RectangleType
beginproc
        movem.l d2-d5/a2-a3,-(a7)                       ; Saves registers

; Initializes buffer
        move.w  Total(a5),d2                            ; Initialize counter
        lea.l   TStr1(a5),a2                            ; Fill Buffer with values from 1 to Total
M_Init  move.b  d2,(a2)+
        dbra    d2,M_Init                               ; While d2 > -1

; Gets random Numbers
        move.w  Total(a5),d3                            ; Will be use for Rnd()
        move.w  Number(a5),d2                           ; Initialize counter
        subi.w  #1,d2
        lea.l   TStr1(a5),a2                            ; Let A2 points to the buffer
M_Rnd
        call    RandInt(d3.w)                           ; Get (in D0.W) a random number from 0 to d3-1
        subi.w  #1,d3                                   ; Decrement it for Rnd()
        move.b  0(a2,d0),d4                             ; Swap the byte-values in (a2) and 0(a2,d0)
        move.b  (a2),0(a2,d0)                           ; ...
        move.b  d4,(a2)                                 ; ...
        addq    #1,a2                                   ; Increment A2
        dbra    d2,M_Rnd                                ; While d2 > -1

; Sort the entire number-serie
        cmpi.w  #1,Number(a5)
        beq.s   M_NoSrt
        call    SortBytes(&TStr1(a5),Number(a5).w)
M_NoSrt
; Converts the entire number-serie to ASCII
        move.w  Number(a5),d2                           ; Initialize counter
        subq    #1,d2
        lea.l   TStr1(a5),a3                            ; Let A3 points to the Bin-buffer
        lea.l   TStr2(a5),a2                            ; Let A2 points to the Ascii-buffer
M_Loop
        moveq   #0,d0                                   ; CLear D0.L
        move.b  (a3)+,d0
        cmpi.b  #9,d0                                   ; Is number > 9 ?
        bgt.s   M_NoZro                                 ; No! Insert a Zero
        move.b  #'0',(a2)+                              ; Insert zero
M_NoZro
        systrap StrIToA(a2.l,d0.l)
        systrap StrLen(a2)                              ; Length of string in D0.W
        adda.w  d0,a2
        move.b  #'-',(a2)+                              ; Insert separator
        dbra    d2,M_Loop                               ; While d2 > -1
        move.b  #0,-(a2)                                ; Insert Null (terminal-string)

        lea.l   TStr2(a5),a2
        systrap StrLen(a2)                              ; Length of string in D0.W
        move.w  d0,d3
        systrap FntCharsWidth(a2.l,d0.w)                ; Length in pixels
        move.w  d0,d4

        move.w  #160,d2                                 ; Screenwidth
        sub.w   d0,d2
        lsr.w   #1,d2                                   ; Div 2
        move.w  d2,d5
        systrap WinDrawChars(a2.l,d3.w,d2.w,#96.w)
        tst.b   Drawn(a5)
        bne     M_Exit
        subi.w  #2,d5
        addi.w  #4,d4                                   ; + 4 pixels
        systrap FntCharHeight()
        addi.w   #4,d0
        call    Rectangle(d5.w,#94.w,d4.w,d0.w,#5.w)
        move.b  #1,Drawn(a5)
M_Exit
        movem.l  (a7)+,d2-d5/a2-a3                      ; Retrieve registers
endproc ; Make()


proc Rectangle(TopLeftX.w,TopLeftY.w,ExtentX.w,ExtentY.w,Art.w)
local Rect.RectangleType
beginproc
        move.w  TopLeftX(a6),Rect.topLeft.x(a6)
        move.w  TopLeftY(a6),Rect.topLeft.y(a6)
        move.w  ExtentX(a6),Rect.extent.x(a6)
        move.w  ExtentY(a6),Rect.extent.y(a6)
        tst.w   Art(a6)
        beq.s   RctErase
        systrap WinDrawRectangleFrame(Art(a6).w,&Rect(a6))
        bra.s   RctExit
RctErase
        systrap WinEraseRectangle(&Rect(a6),#0.w)
;        systrap WinFillRectangle(&Rect(a6),#0.w)
RctExit
endproc ; Rectangle()


; Return (in D0.W) a random number from 0 to Limit-1
proc RandInt(Limit.w)
beginproc
        move.l  #0,-(a7)
        trap    #15
        dc.w    sysTrapSysRandom
        addq.l  #4,a7
        divu    Limit(a6),d0                            ; Divide by Number
        clr.w   d0                                      ; Clear quotient
        swap    d0                                      ; Get rest (a number between 0 and Number-1)
endproc ; RandInt()


; Sort Elem bytes at adresse BufPtr
proc SortBytes(BufPtr.l,Elem.w)
beginproc
        movem.l d1-d3/a1-a3,-(a7)                       ; Saves registers
        move.w  Elem(a6),d1                             ; Initialize 1st loop counter
        subi.w  #2,d1
        movea.l BufPtr(a6),a1                           ; Let's A1 points to the Byte-buffer
SrtB_1  movea.l a1,a2                                   ; ---  ... A2 to
        movea.l a2,a3                                   ;    | ... and A3 to
        move.b  (a2),d3                                 ;    | Byte from (a2) in D3
        move.w  d1,d2                                   ;    | Initialize 2nd loop-counter
SrtB_2  add.l   #1,a3                                   ; -  | Increment A3
        cmp.b   (a3),d3                                 ;  | | Compare Byte in (A3) with Byte in (A2)
        blt.s   SrtB_3                                  ;  | | Smaller ?
        movea.l a3,a2                                   ;  | | Yes. A2 = A3
        move.b  (a2),d3                                 ;  | | Byte from (a2) in D3
SrtB_3  dbra    d2,SrtB_2                               ; -  | While d2 > -1
        cmpa.l  a1,a2                                   ;    |
        beq.s   SrtB_4                                  ;    |
        move.b  (a1),(a2)                               ;    | Swap the bytes in (A1) and (A2)
SrtB_4  move.b  d3,(a1)+                                ;    | ... and increment A1
        dbra    d1,SrtB_1                               ; --- While d2 > -1
        movem.l  (a7)+,d1-d3/a1-a3                      ; Retrieve registers
endproc ; SortBytes()


;
; Resources -----------------------------------------------------------------
;

; Application Icon Bitmap resource. Is automatically converted from Windows
; format to Pilot format and written as a 'tAIB' rather than 'Tbmp' because
; kidbAIB is a special value ($7ffe)

        res 'WBMP', kidbAIB, "Lotto.bmp"

; 'pref' resource. Defines app launch flags, stack and heap size

        res 'pref', kidrPREF
        dc.w    sysAppLaunchFlagNewStack|sysAppLaunchFlagNewGlobals|sysAppLaunchFlagUIApp|sysAppLaunchFlagSubCall
        dc.l    $1000                           ; stack size
        dc.l    $1000                           ; heap size

        include "Lotto.res"  ; Pilrc generated resources

        end
