/***********************************************************************
 *
 *	Copyright  Jeff Jetton 1997 -- All Rights Reserved
 *
 * PROJECT:			Pilot Tap Tester Application: Be the first kid on your block
 *						to document the new bug on your PalmPilot Personal!!!
 *
 * FILE:				TapTest.c
 *
 *
 **********************************************************************/
 
#include <Pilot.h>				// all the system toolbox headers

#define	mainForm			1000	// Main form for the application
#define	stringX			80		// Position of the drawn coordinates
#define	stringY			1
#define	markHeight		3		//	The size of the mark made to show the pendown
#define	markWidth		3
#define	titleBarBottom	15		// The bottom of the title bar. Duh!
#define	noPreviousMark	999


/***********************************************************************
 * Prototypes for internal functions
 **********************************************************************/
static void StartApplication(void);
static void StopApplication(void);
static void PenDown( SWord, SWord );
static void DrawCoordinates( SWord x, SWord y );
static Boolean MainFormHandleEvent(EventPtr);			// handle event for main form
static Boolean ApplicationHandleEvent(EventPtr);		// handle form load events
static void EventLoop(void);

/***********************************************************************
 * Globals for this module
 **********************************************************************/

static SWord	MarkX, MarkY;	// The previous pen location


/***********************************************************************
 *
 * FUNCTION:     StartApplication
 *
 * DESCRIPTION:  This routine sets up the initial state of the application
 *               and sets up global variables.
 *
 * PARAMETERS:   None.
 *
 * RETURNED:     nothing.
 *
 ***********************************************************************/
static void StartApplication(void)
{	
	/* The ParsePenDown rountine first erases the previous mark.
		Set MarkX so we'll know not to do that yet. */
		
	MarkX = noPreviousMark;
}


/***********************************************************************
 *
 * FUNCTION:    StopApplication
 *
 * DESCRIPTION: This routine cleans up stuff
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void StopApplication(void)
{
	// Nothing to do, really.
}



/***********************************************************************
 *
 * FUNCTION:    PenDown
 *
 * DESCRIPTION: Draws a mark at the pen coordinates and calls DrawCoordinates()
 *
 * PARAMETERS:  the x and y coordinates of the pendown
 *
 * RETURNED:    not a dang thing
 *
 ***********************************************************************/
static void PenDown( SWord x, SWord y )
{
	RectangleType	bounds;
	
	
	// Set up the mark extents
	bounds.extent.x = markWidth;
	bounds.extent.y = markHeight;
	
	// Erase the previous mark only if there is a previous mark to erase
	if ( MarkX != noPreviousMark )
	{
		bounds.topLeft.x = MarkX - 1;
		bounds.topLeft.y = MarkY - 1;
		WinInvertRectangle( &bounds, 0);
	}
	
	// Set the Mark coords to the new pen coords
	MarkX = x;
	MarkY = y;
	
	// Draw the new coordinates
	DrawCoordinates( x, y );
	
	// Draw the new mark. Since the mark is 3 pixels square, adjust the position
	// so that it's centered over the actual coordinates. 
	bounds.topLeft.x = MarkX - 1;
	bounds.topLeft.y = MarkY - 1;
	WinInvertRectangle( &bounds, 0);
	
	// Click
	SndPlaySystemSound( sndClick );
}


/***********************************************************************
 *
 * FUNCTION:    DrawCoordinates
 *
 * DESCRIPTION: Draws the coordinates at the top of the screen
 *
 * PARAMETERS:  the x and y coordinates of the pendown
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void DrawCoordinates( SWord x, SWord y )
{
	RectangleType	bounds;
	Char				text[10], moreText[10];
	FontID			yeOldeFont;
	
	// Remember the current font and set new font
	// (Mac programming habits die hard)
	yeOldeFont = FntSetFont( boldFont );
	
	// Set up a string using the passed coordinates
	StrIToA( text, x );
	StrCat( text, ", " );
	StrIToA( moreText, y );
	StrCat( text, moreText );
	
	// Erase the old coordinates
	bounds.topLeft.x = stringX;
	bounds.topLeft.y = stringY;
	bounds.extent.x = 160 - stringX;
	bounds.extent.y = 12;
	WinEraseRectangle( &bounds, 0 );
	
	// Display the new coordinates
	WinDrawChars( text, StrLen(text), stringX, stringY );
}

/***********************************************************************
 *
 * FUNCTION:		MainFormHandleEvent
 *
 * DESCRIPTION:	Handles processing of events for the main form.
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr event)
{
	Boolean		handled = false;

	switch (event->eType)
	{
		
  		case frmOpenEvent:	// The form was told to open.
  			// It has already been loaded and activated so just draw it.
			FrmDrawForm (FrmGetActiveForm());
			handled = true;
			break;
			
		case penDownEvent:    // the pen was pressed to the screen
			PenDown( event->screenX, event->screenY );
			handled = true;
			break;
	}

	return(handled);
}


/***********************************************************************
 *
 * FUNCTION:    ApplicationHandleEvent
 *
 * DESCRIPTION: This routine loads a form resource and sets the event handler for the form.
 *
 * PARAMETERS:  event - a pointer to an EventType structure
 *
 * RETURNED:    True if the event has been handled and should not be
 *						passed to a higher level handler.
 *
 ***********************************************************************/
static Boolean ApplicationHandleEvent(EventPtr event)
{
	FormPtr	frm;
	Word		formId;
	Boolean	handled = false;

	if (event->eType == frmLoadEvent)
	{
		// Load the form resource specified in the event then activate the form.
		formId = event->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);

		// Set the event handler for the form.  The handler of the currently 
		// active form is called by FrmDispatchEvent each time it receives an event.
		// The about form is dealt with by the DoDialog function.
		switch (formId)
		{
			case mainForm:
				FrmSetEventHandler(frm, MainFormHandleEvent);
				break;
		}
		handled = true;
	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:		EventLoop
 *
 * DESCRIPTION:	A simple loop that obtains events from the Event
 *						Manager and passes them on to various applications and
 *						system event handlers before passing them on to
 *						FrmDispatchEvent for default processing.
 *
 * PARAMETERS:		None.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void EventLoop(void)
{
	EventType	event;
	Word			error;
	
	do
		{
		// Get the next available event.
		EvtGetEvent(&event, evtWaitForever);
		
		// Give the system a chance to handle the event.
		if (! SysHandleEvent(&event))

			// Give the menu bar a chance to update and handle the event.	
			if (! MenuHandleEvent(0, &event, &error))

				// Give the application a chance to handle the event.
				if (! ApplicationHandleEvent(&event))

					// Let the form object provide default handling of the event.
					FrmDispatchEvent(&event);
		}
	while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:		PilotMain
 *
 * DESCRIPTION:	This function is the equivalent of a main() function
 *						under standard C.  It is called by the Emulator to begin
 *						execution of this application.
 *
 * PARAMETERS:		cmd - command specifying how to launch the application.
 *						cmdPBP - parameter block for the command.
 *						launchFlags - flags used to configure the launch.			
 *
 * RETURNED:		Any applicable error codes.
 *
 ***********************************************************************/
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	// This app doesn't support any action codes.
	if (cmd != sysAppLaunchCmdNormalLaunch) return sysErrParamErr;
	
	// Initialize the application's global variables and database.
	StartApplication();
   
	// Start the first form.
	FrmGotoForm(mainForm);
		
	// Start the event loop.
	EventLoop();
	
	// Clean up before exiting the applcation.
	StopApplication();
	
	return 0;
}
