/***********************************************************************
 *
 Copyright  1995 - 1998, 3Com Corporation or its subsidiaries ("3Com").  
 All rights reserved.
   
 This software may be copied and used solely for developing products for 
 the Palm Computing platform and for archival and backup purposes.  Except 
 for the foregoing, no part of this software may be reproduced or transmitted 
 in any form or by any means or used to make any derivative work (such as 
 translation, transformation or adaptation) without express written consent 
 from 3Com.

 3Com reserves the right to revise this software and to make changes in content 
 from time to time without obligation on the part of 3Com to provide notification 
 of such revision or changes.  
 3COM MAKES NO REPRESENTATIONS OR WARRANTIES THAT THE SOFTWARE IS FREE OF ERRORS 
 OR THAT THE SOFTWARE IS SUITABLE FOR YOUR USE.  THE SOFTWARE IS PROVIDED ON AN 
 "AS IS" BASIS.  3COM MAKES NO WARRANTIES, TERMS OR CONDITIONS, EXPRESS OR IMPLIED, 
 EITHER IN FACT OR BY OPERATION OF LAW, STATUTORY OR OTHERWISE, INCLUDING WARRANTIES, 
 TERMS, OR CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND 
 SATISFACTORY QUALITY.

 TO THE FULL EXTENT ALLOWED BY LAW, 3COM ALSO EXCLUDES FOR ITSELF AND ITS SUPPLIERS 
 ANY LIABILITY, WHETHER BASED IN CONTRACT OR TORT (INCLUDING NEGLIGENCE), FOR 
 DIRECT, INCIDENTAL, CONSEQUENTIAL, INDIRECT, SPECIAL, OR PUNITIVE DAMAGES OF 
 ANY KIND, OR FOR LOSS OF REVENUE OR PROFITS, LOSS OF BUSINESS, LOSS OF INFORMATION 
 OR DATA, OR OTHER FINANCIAL LOSS ARISING OUT OF OR IN CONNECTION WITH THIS SOFTWARE, 
 EVEN IF 3COM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

 3Com, HotSync, Palm Computing, and Graffiti are registered trademarks, and 
 Palm III and Palm OS are trademarks of 3Com Corporation or its subsidiaries.

 IF THIS SOFTWARE IS PROVIDED ON A COMPACT DISK, THE OTHER SOFTWARE AND 
 DOCUMENTATION ON THE COMPACT DISK ARE SUBJECT TO THE LICENSE AGREEMENT 
 ACCOMPANYING THE COMPACT DISK.

 ********************************************************************************
 *
 * PROJECT:			Memo Pad Sample Application
 *
 * FILE:				MemoPad.c (Phase 11)
 *
 *		For best formatting set tab size to 3.
 *
 * DESCRIPTION:	Sample application demonstrating categories.
 *	  
 * OBJECTIVE:		Add categories.
 *
 * REVISION HISTORY:
 * 	 1/8/96	Roger		Initial version
 *
 **********************************************************************/
 
#include <Pilot.h>				// all the system toolbox headers
#include <FeatureMgr.h>			//	P4. Needed to get the ROM version
#include "MemoPadRsc.h"			// application resource defines


/***********************************************************************
 * Structures for this module
 **********************************************************************/

// P11. An application info structure is intended to be stored in the
// database's app info block and to contain data relevant to the database.
// For now we store just category information.  Category information is 
// always placed in the beginning where the category.c code expects it.
typedef struct 
	{
   UInt	renamedCategories;		// bitfield of categories with a changed name
	char	categoryLabels[dmRecNumCategories][dmCategoryLength];
	Byte	categoryUniqIDs[dmRecNumCategories];
	Byte	lastUniqID;			// Uniq IDs for categories generated by the device 
									// are between 0 - 127.  Those from the PC are 128 - 255.
	Byte	reserved1;			// from the compiler word aligning things
	Word	reserved2;
	} MemoPadAppInfoType;

typedef MemoPadAppInfoType * MemoPadAppInfoPtr;		// P11.


/***********************************************************************
 * Global defines for this module
 **********************************************************************/
#define version20	0x02000000	// P4. PalmOS 2.0 version number

// Application database management defines.
#define memoPadAppType		'Memo'			// P5. type for application.  must be 4 chars, mixed case.
#define memoPadDBType		'Data'			// P5. type for application database.  must be 4 chars, mixed case.
#define memoPadDBName		"MemoPadDB"		// P5. name for application database.  up to 31 characters.
#define noRecordSelected	-1					// P5. index for no current record.
#define newMemoSize  		64					// P6. initial size for new database record.

#define updateCategoryChanged		0x01		// P11. Indicates a category changed
#define localizedAppInfoStr		1000		// P11. default category strings ** tAIS **


/***********************************************************************
 * Global variables for this module
 **********************************************************************/
static Word				CurrentView;			// P3. id of current form
static DmOpenRef		MemoPadDB;				// P5. handle for database
static Word				CurrentRecord;			// P5. index of current database record
static VoidHand 		ChoicesHandle = 0, ChoicesPtrsHandle;	// P7. handles for list data
static char				CategoryName [dmCategoryLength];		// P11. Storage for the name
static Word				CurrentCategory = dmAllCategories;	// P11. category filter
static Word				RecordCategory;		// P11. edited record's category (never All)
static Word 			NewCategory;			// P11. category in details form (never All)
static Boolean			NewCategoryEdited;	// P11. true if name edited


/***********************************************************************
 * Prototypes for internal functions
 **********************************************************************/
static void StartApplication(void);
static void StopApplication(void);								// P5. clean up before app exit
static VoidPtr GetObjectPtr(Word objectID);					// P10. get ptr to object in active form
static FieldPtr GetFocusObjectPtr(void);						// P4. get ptr to text field
static Boolean CreateRecord(void);								// P6. create a new record in the application database
static void DetailsInit(void);									// P10. initialize data, objects for the details form
static UInt DetailsApply(void);									// P10. apply changes made in the details form
static Boolean DetailsFormHandleEvent(EventPtr event);	// P10. handle events for the details form
static void EditViewDoMenuCommand(Word command);					// P4. perform commands for edit form menu
static Boolean EditViewDeleteCurrentRecord(void);				// P10. delete database record
static void EditViewSaveData(FieldPtr fld);						// P5. save data to application database
static void EditViewRetrieveData(FieldPtr fld);					// P5. get data from application database
static Boolean EditViewHandleEvent(EventPtr event);
static void MainViewListDrawItem(UInt itemNum, 
					RectanglePtr bounds, CharPtr *itemsText);	// P8. draw an item in the main list
static void MainViewInit(void);									// P7. initialize data, objects for the main form
static Boolean MainViewHandleEvent(EventPtr event);
static Boolean ApplicationHandleEvent(EventPtr event);	// P3. handle form load events
static void EventLoop(void);


/************************************************************
 *
 *  FUNCTION: MemoPadInitAppInfo
 *
 *  DESCRIPTION: P11. Create an app info chunk if missing.  Set
 *		the category strings to a default.
 *
 *  PARAMETERS: database pointer
 *
 *  RETURNS: 0 if successful, errorcode if not
 *
 *************************************************************/
static Err	MemoPadInitAppInfo(DmOpenRef dbP)
{
	UInt cardNo;
	VoidHand h;
	LocalID dbID;
	LocalID appInfoID;
	MemoPadAppInfoPtr	appInfoP;
	MemoPadAppInfoPtr	nilP = 0;

	
	// We have a DmOpenRef and we want the database's app info block ID.
	// Get the database's dbID and cardNo and then use them to get the appInfoID.
	if (DmOpenDatabaseInfo(dbP, &dbID, NULL, NULL, &cardNo, NULL))
		return dmErrInvalidParam;
	if (DmDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, 
			NULL, NULL, &appInfoID, NULL, NULL, NULL))
		return dmErrInvalidParam;


	// If no appInfoID exists then we must create a new one.	
	if (appInfoID == NULL)
		{
		h = DmNewHandle (dbP, sizeof (MemoPadAppInfoType));
		if (! h) return dmErrMemError;
		
		appInfoID = MemHandleToLocalID (h);
		DmSetDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &appInfoID, NULL, NULL, NULL);
		}


	// Lock the appInfoID and copy in defaults from our default structure.	
	appInfoP = MemLocalIDToLockedPtr(appInfoID, cardNo);

	// Clear the app info block.
	DmSet (appInfoP, 0, sizeof (MemoPadAppInfoType), 0);

	// Initialize the categories.
	CategoryInitialize ((AppInfoPtr) appInfoP, localizedAppInfoStr);

	// Initialize reserved2.  This is actually redundant since the DmSet
	// call above has already set reserved2 to zero.  But this shows how
	// to set variables in addition to the categories.
	DmSet (appInfoP, (ULong)&nilP->reserved2, sizeof(appInfoP->reserved2), 
		0);

	MemPtrUnlock(appInfoP);
	return 0;
}


/***********************************************************************
 *
 * FUNCTION:     StartApplication
 *
 * DESCRIPTION:  This routine sets up the initial state of the application.
 *               It opens the application's database and sets up global variables. 
 *
 * PARAMETERS:   None.
 *
 * RETURNED:     nothing.
 *
 ***********************************************************************/
static void StartApplication(void)
{
	Word		error;			// P5. error code
	UInt		cardNo;			// P5. card containing the application database
	LocalID	dbID;				// P5. handle for application database
	UInt		dbAttrs;			// P5. database attributes
	UInt		mode;				// P5. permissions when opening the database

	// P10. Determime if secret record should be shown.
	if ( PrefGetPreference(prefHidePrivateRecords) )
		mode = dmModeReadWrite;
	else
		mode = dmModeReadWrite | dmModeShowSecret;

	// P5. Find the application's database.
	MemoPadDB = DmOpenDatabaseByTypeCreator(memoPadDBType, memoPadAppType, mode);
	if (! MemoPadDB)
		{
		// P5. The database doesn't exist, create it now.
		error = DmCreateDatabase(0, memoPadDBName, memoPadAppType, memoPadDBType, false);

		// P5. Check for fatal error.
		ErrFatalDisplayIf(error, "Could not create new database.");
		
		// P5. Find the application's database.
		MemoPadDB = DmOpenDatabaseByTypeCreator(memoPadDBType, memoPadAppType, mode);

		// P5. Get info about the database
		error = DmOpenDatabaseInfo(MemoPadDB, &dbID, NULL, NULL, &cardNo, NULL);

		// P5. Check for fatal error.
		ErrFatalDisplayIf(error, "Could not get database info.");
	
		// P5. Get attributes for the database
		error = DmDatabaseInfo(0, dbID, NULL, &dbAttrs, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

		// P5. Check for fatal error.
		ErrFatalDisplayIf(error, "Could not get database info.");
	
		// P5. Set the new attributes in the database
		error = DmSetDatabaseInfo(0, dbID, NULL, &dbAttrs, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

		// P5. Check for fatal error.
		ErrFatalDisplayIf(error, "Could not set database info.");
		
		// P11. Store category info in the application's information block.
		MemoPadInitAppInfo(MemoPadDB);
		}
	
	// P5. Set current database record to no current record.
	CurrentRecord = noRecordSelected;

	// P11. Get the name of the current category from the app info block.
	// The app info block was created and initialized by MemoPadInitAppInfo.
	CategoryGetName (MemoPadDB, CurrentCategory, CategoryName);

	// P4. Set current view so the first form to display is the main form.
	CurrentView = MemoPadMainForm;
}


/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: P4. Check that the ROM version meets your
 *              minimum requirement.  Warn if the app was switched to.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags indicating how the application was
 *											 launched.  A warning is displayed only if
 *                                these flags indicate that the app is 
 *											 launched normally.
 *
 * RETURNED:    zero if rom is compatible else an error code
 *                             
 ***********************************************************************/
static Err RomVersionCompatible (DWord requiredVersion, Word launchFlags)
{
	DWord romVersion;
	
	
	// See if we're on in minimum required version of the ROM or later.
	// The system records the version number in a feature.  A feature is a
	// piece of information which can be looked up by a creator and feature
	// number.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		// If the user launched the app from the launcher, explain
		// why the app shouldn't run.  If the app was contacted for something
		// else, like it was asked to find a string by the system find, then
		// don't bother the user with a warning dialog.  These flags tell how
		// the app was launched to decided if a warning should be displayed.
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Pilot 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.  The sysFileCDefaultApp is considered "safe".
			if (romVersion < 0x02000000)
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		
		return (sysErrRomIncompatible);
		}

	return 0;
}

/***********************************************************************
 *
 * FUNCTION:    StopApplication
 *
 * DESCRIPTION: P5. This routine closes the application's database.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void StopApplication(void)
{
	// P11. Close all open forms to allow their frmCloseEvent handlers
	// to execute.  An appStopEvent doesn't send frmCloseEvents.
	// FrmCloseAllForms will send all opened forms a frmCloseEvent.
	FrmCloseAllForms ();

	// Close the application's database.
	DmCloseDatabase(MemoPadDB);
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: P10. This routine returns a pointer to an object in the active form.
 *
 * PARAMETERS:  objectID - id of the object
 *
 * RETURNED:    pointer to the object's data structure
 *
 ***********************************************************************/
static VoidPtr GetObjectPtr(Word objectID)
{
	FormPtr frm;
	
	frm = FrmGetActiveForm();
	return(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectID)));
}

/***********************************************************************
 *
 * FUNCTION:    GetFocusObjectPtr
 *
 * DESCRIPTION: P4. This routine returns a pointer to the field object, in 
 *              the current form, that has the focus.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    pointer to a field object or NULL of there is no focus
 *
 ***********************************************************************/
static FieldPtr GetFocusObjectPtr(void)
{
	FormPtr frm;
	Word focus;
	
	// Get a pointer to tha active form and the index of the form object with focus.
	frm = FrmGetActiveForm();
	focus = FrmGetFocus(frm);
	
	// If no object has the focus return NULL pointer.
	if (focus == noFocus)
		return(NULL);
		
	// Return a pointer to the object with focus.
	return(FrmGetObjectPtr(frm, focus));
}


/***********************************************************************
 *
 * FUNCTION:    ChangeCategory
 *
 * DESCRIPTION: P11. Updates the edit view and it's variables.  
 *
 * PARAMETERS:  category  - new category (index)
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void ChangeCategory (Word category)
{
	// Remember what category the record is in.
	RecordCategory = category;
	
	// The MemoPadEditForm should be updated to display the new category.
	// Send a frmUpdateEvent with our updateCategoryChanged code
	// which we defined. 
	FrmUpdateForm (MemoPadEditForm, updateCategoryChanged);
}


/***********************************************************************
 *
 * FUNCTION:    CreateRecord
 *
 * DESCRIPTION: P6. This routine creates a new memo record.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    true if the record was sucessfully created.
 *
 ***********************************************************************/
static Boolean CreateRecord(void)
{
	Ptr		p;
	VoidHand memoRec;
	UInt		index = 0;
	Err		error;
	Char		zero = 0;
	UInt		attr;			// P11. record attributes (category)

	// Create a new first record in the database
	memoRec = DmNewRecord(MemoPadDB, &index, newMemoSize);

	// Lock down the block containing the new record.
	p = MemHandleLock(memoRec);

	// Write a zero to the first byte of the record to null terminate the new memo string.
	error = DmWrite(p, 0, &zero, sizeof(Char));
	
	// Check for fatal error.
	ErrFatalDisplayIf(error, "Could not write to new record.");
	
	// Unlock the block of the new record.
	MemPtrUnlock(p);


	// P11. Set the category of the new record to the current category.
	DmRecordInfo (MemoPadDB, index, &attr, NULL, NULL);
	attr &= ~dmRecAttrCategoryMask;				// Remove all category bits
	if (CurrentCategory == dmAllCategories)
		{
		// Since the user isn't looking in any particular category place
		// the new record in the unfiled category.
		attr |= dmUnfiledCategory;
		}
	else
		{
		// Set the attribute bits to indicate the current category.
		attr |= CurrentCategory;
		}
	DmSetRecordInfo (MemoPadDB, index, &attr, NULL);


	// Release the record to the database manager.  The true value indicates that
	// the record contains "dirty" data.  Release Record will set the record's 
	// dirty flag and update the database modification count.
	DmReleaseRecord(MemoPadDB, index, true);

	// Remember the index of the current record.
	CurrentRecord = 0;
	return(true);	
}


/***********************************************************************
 *
 * FUNCTION:    DetailsApply
 *
 * DESCRIPTION: P10. This routine applies the changes made in the 
 *              details form to the current record.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    P11. code which indicates how the memo was changed
 *					 the code will be sent as the update code, in a frmUpdate event
 *
 ***********************************************************************/
static UInt DetailsApply(void)
{
	UInt		attr;
	Boolean	secret;
	UInt		updateCode = 0;
	Boolean	dirty = false;		// P11. If an attribute changed
	
	// Get the secret attribute of the current record.
	DmRecordInfo(MemoPadDB, CurrentRecord, &attr, NULL, NULL);

	// Get the current setting of the secret checkbox.
	secret = (CtlGetValue(GetObjectPtr(MemoPadDetailsPrivateCheckbox)) != 0);

	// Has the secret attribute been changed?
	if (((attr & dmRecAttrSecret) == dmRecAttrSecret) != secret)
		{
		// Yes, it was changed.  What is the new setting?
		if (secret)
			// Now it should be secret.
			attr |= dmRecAttrSecret;
		else
			// Now it should not be secret.
			attr &= ~dmRecAttrSecret;

		dirty = true;
		}


	// P11. Has the category been changed?
	if (RecordCategory != NewCategory)
		{
		attr &= ~dmRecAttrCategoryMask;		// Remove all category bits.
		attr |= NewCategory;						// Set the new category
		dirty = true;								// mark this record dirty later
		RecordCategory = NewCategory;			// remember which category the record is in
		updateCode |= updateCategoryChanged;	// send this update code
		}
	

	// P11. If the current category was deleted, renamed, or merged with
	// another category, then the list view needs to be redrawn.
	if (NewCategoryEdited)
		{
		RecordCategory = NewCategory;
		updateCode |= updateCategoryChanged;
		}


	if (dirty)
		{
		// Set the dirty flag for the record.
		attr |= dmRecAttrDirty;
		DmSetRecordInfo (MemoPadDB, CurrentRecord, &attr, NULL);
		}
	
	return updateCode;
}


/***********************************************************************
 *
 * FUNCTION:    DetailsInit
 *
 * DESCRIPTION: P10. This routine initializes the Details Dialog.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void DetailsInit(void)
{
	ControlPtr	ctl;
	UInt			attr;	
	Boolean 		secret;
	
	
	// P11. NewCategory indicates which category wants to changed the 
	// current record to.  Initialize it to the current record's current
	// category.
	NewCategory = RecordCategory;
	NewCategoryEdited = false;
			
	// Get the secret and category attributes of the current record.
	DmRecordInfo(MemoPadDB, CurrentRecord, &attr, NULL, NULL);
	secret = attr & dmRecAttrSecret;

	// If the record is marked secret, turn on the secret checkbox.
	CtlSetValue(GetObjectPtr(MemoPadDetailsPrivateCheckbox), secret);

	// P11. Set the label of the category trigger.
	CategoryGetName (MemoPadDB, RecordCategory, CategoryName);
	ctl = GetObjectPtr (MemoPadDetailsCategoryPopTrigger);
	CategorySetTriggerLabel (ctl, CategoryName);
}


/***********************************************************************
 *
 * FUNCTION:    DetailsFormHandleEvent
 *
 * DESCRIPTION: P10. Handles processing of events for the details form.
 *
 * PARAMETERS:  event - the most recent event.
 *
 * RETURNED:    True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean DetailsFormHandleEvent(EventPtr event)
{
	FormPtr		frm;
	Boolean		handled = false;
	Word 			updateCode;				// P11. updates for the edit form

	if (event->eType == ctlSelectEvent)
		{
   	// A control button was pressed and released.	
		switch (event->data.ctlSelect.controlID)
			{
			case MemoPadDetailsOKButton:
	   		// The ok button was pressed so apply the changes.
				updateCode = DetailsApply();
				
				// Remove the details form and display the edit form.
				FrmReturnToForm(MemoPadEditForm);
				if (updateCode)
					FrmUpdateForm (MemoPadEditForm, updateCode);
				handled = true;
				break;

			case MemoPadDetailsCancelButton:
	   		// The cancel button was pressed, just go back to the edit form.
				FrmReturnToForm(MemoPadEditForm);
				handled = true;
				break;
				
			case MemoPadDetailsDeleteButton:
	   		// The delete button was pressed.  Delete the current record.
				if (EditViewDeleteCurrentRecord())
					{
					// Delete was completed.  Close the details and the edit forms.
					FrmCloseAllForms();
					
					// Go to the main form.
					FrmGotoForm(MemoPadMainForm);
					}
				handled = true;
				break;
			
			case MemoPadDetailsCategoryPopTrigger:
				// P11. The category trigger was pressed.  Have the
				// system ui present a list of categories and return
				// what the user selects.  The categories listed are those
				// found in the app info block.
				if (CategorySelect (MemoPadDB, FrmGetActiveForm (),
					MemoPadDetailsCategoryPopTrigger, MemoPadDetailsCategoriesListList,
					false, &NewCategory, CategoryName, 1, 0))
					{
					NewCategoryEdited = true;
					}
				handled = true;
				break;

			}
		}

	else if (event->eType == frmOpenEvent)
		{
  		// The form was told to open.  It has already been loaded and activated.
		frm = FrmGetActiveForm();
		
  		// Initialize the details form.
		DetailsInit();
		
		// Draw the details form.
		FrmDrawForm(frm);
		handled = true;
		}

	return(handled);
}


/***********************************************************************
 *
 * FUNCTION:    EditViewUpdateDisplay
 *
 * DESCRIPTION: P11. This routine updates the display of the edit view
 *
 * PARAMETERS:  updateCode - a code that indicated what changes have been
 *                           made to the view.
 *                		
 * RETURNED:    nothing
 *
 ***********************************************************************/
static Boolean EditViewUpdateDisplay (Word updateCode)
{
	FormPtr frm;

	frm = FrmGetActiveForm ();

	if (updateCode == updateCategoryChanged)
		{
		// Set the label of the category trigger.
		CategoryGetName (MemoPadDB, RecordCategory, CategoryName);
		CategorySetTriggerLabel (GetObjectPtr (MemoPadEditCategoryPopTrigger),
			CategoryName);
		
		// If the record won't be visible in the main form change the
		// category to keep it visible.  If the user doesn't see the 
		// record they might fear it lost.
		if (CurrentCategory != dmAllCategories &&
			CurrentCategory != RecordCategory)
			CurrentCategory = RecordCategory;
			
		return (true);
		}
	
	return (false);
}


/***********************************************************************
 *
 * FUNCTION:    EditViewSelectCategory
 *
 * DESCRIPTION: P11. This routine changes the category of the memo 
 * 				 in the edit form.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 *              The following global variables are modified:
 *							CurrentCategory
 *							CategoryName
 *
 ***********************************************************************/
static void EditViewSelectCategory (void)
{
	UInt attr;
	FormPtr frm;
	UInt category;
	Boolean categorySelected;
	Boolean categoryEdited;


	// Get the current category.	
	category = RecordCategory;
	
	// Process the category popup list.
	frm = FrmGetActiveForm();
	
	// Have the system ui present the user with a list of categories
	// and return what the user selects.
	categoryEdited = CategorySelect (MemoPadDB, frm, MemoPadEditCategoryPopTrigger,
					  MemoPadEditCategoriesListList, false, &category, CategoryName, 1, 0);


	categorySelected = (category != RecordCategory);

	// If a different category was selected, set the category field
	// in the new record.
	if (categorySelected)
		{
		// Change the category of the record.
		DmRecordInfo (MemoPadDB, CurrentRecord, &attr, NULL, NULL);	
		attr &= ~dmRecAttrCategoryMask;
		attr |= (category | dmRecAttrDirty);
		DmSetRecordInfo (MemoPadDB, CurrentRecord, &attr, NULL);
		}


	// If the current category was changed or the name of the category 
	// was edited,  draw the title.
	if (categoryEdited || categorySelected)
		{
		ChangeCategory (category);
		}
}


/***********************************************************************
 *
 * FUNCTION:    EditViewDeleteCurrentRecord
 *
 * DESCRIPTION: P10. This routine deletes the current memo record following
 *              confirmation from the user.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    True if the record is deleted, false otherwise.
 *
 ***********************************************************************/
static Boolean EditViewDeleteCurrentRecord(void)
{
	FormPtr  frm;
	FieldPtr fld;

	// Display alert to get confirmation for the delete request.
	if (FrmAlert(DeleteAlertAlert) == DeleteAlertOK)
		{
		// The delete has been confirmed.
		// Get a pointer to the field.
		frm = FrmGetFormPtr(MemoPadEditForm);
		fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MemoPadEditEditFieldField));
		
		// Delete the text in the field, if there is any.
		if (FldGetTextPtr(fld))
			FldDelete(fld, 0, FldGetTextLength(fld));
			
		// Inform the caller that the delete was completed.
		return(true);
		}
	
	else 
		// Inform the caller that the delete was cancelled.
		return(false);
}


/***********************************************************************
 *
 * FUNCTION:		EditViewSaveData
 *
 * DESCRIPTION:	P5. Saves the data in the field as the first record in 
 *                the application's database.
 *
 * PARAMETERS:		fld - the field that contains the text.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void EditViewSaveData(FieldPtr fld)
{
	CharPtr	text;
	
	// Get a pointer to the field's text.
	text = FldGetTextPtr(fld);

	// P6. Clear the handle value in the field, otherwise the handle
	// will be freed when the form is closed.
	// This will cause problems since the handle is to the data
	// IN the database record, not a separate copy.
	FldSetTextHandle(fld, 0);

	// Is there data in the field?
	if (text != NULL && *text != 0)
		{
		// Release the record to the database manager.  The true value indicates that
		// the record contains "dirty" data.  Release Record will set the record's 
		// dirty flag and update the database modification count.
		DmReleaseRecord(MemoPadDB, CurrentRecord, true);
		}
	else 
		{
		// P7. The record is empty, remove it from the database.
		DmRemoveRecord(MemoPadDB, CurrentRecord);
		CurrentRecord = noRecordSelected;
		}
}


/***********************************************************************
 *
 * FUNCTION:		EditViewRetrieveData
 *
 * DESCRIPTION:	P5. Retrieves the data from the first record of a database 
 *						and places it in a field.
 *
 * PARAMETERS:		fld - the field to contain the text.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void EditViewRetrieveData(FieldPtr fld)
{
	VoidHand	recHandle;
	UInt attr;
	
	
	// Check to make sure there is a record to retrieve.
	if (CurrentRecord != noRecordSelected && DmNumRecords(MemoPadDB) > 0)
		{
		// Get a handle for the first record.
		recHandle = DmGetRecord(MemoPadDB, CurrentRecord);
		
		// P6. Set the field's text to the data in the record.
		FldSetTextHandle(fld, recHandle);

		// P11. Set the global variable that keeps track of the current category
		// to the category of the current record.
		DmRecordInfo (MemoPadDB, CurrentRecord, &attr, NULL, NULL);
		RecordCategory = attr & dmRecAttrCategoryMask;
		
		// P11. Set the label that contains the note's category.
		CategoryGetName (MemoPadDB, RecordCategory, CategoryName);
		CategorySetTriggerLabel (GetObjectPtr (MemoPadEditCategoryPopTrigger), CategoryName);
	
		}
}


/***********************************************************************
 *
 * FUNCTION:    EditViewDoMenuCommand
 *
 * DESCRIPTION: P4. This routine performs the menu command specified.
 *
 * PARAMETERS:  command - menu item id
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void EditViewDoMenuCommand(Word command)
{
	FieldPtr	fld;
	FormPtr	frm;

	// Do the appropriate action for the menu command selected.
	switch (command)
		{
		case EditPageNewPage:		// P9. Create a new memo page.
			// Save the current data in the field.
			fld = GetFocusObjectPtr();
			if (fld)
			{
   			EditViewSaveData(fld);

				// Clear the existing text in the field.
  				FldDelete(fld, 0, FldGetTextLength(fld));
   			
				// Create a new record in the database.
				if (CreateRecord())
					{
	   			// Get the data for the field from the application's database.
   				EditViewRetrieveData(fld);
   			
   				// Draw the new field data
   				FldDrawField(fld);
   				}
   			}
			break;
		
		case EditPageDeletePage:	// P9. Delete the current memo page.
			// P10. Delete the current database record.
			if (EditViewDeleteCurrentRecord())
	   		// Go back to the main form.
				FrmGotoForm(MemoPadMainForm);
			break;
			
		case EditOptionsGetInfo:		// Display the info form.
			// Load the info form, then display it.
			frm = FrmInitForm(MemoPadInfoForm);
			FrmDoDialog(frm);
			
			// Delete the info form.
	 		FrmDeleteForm(frm);
			break;
		
		case EditOptionsGototopofpage:		// Go to the top of the field.	
			fld = GetFocusObjectPtr();
			if (fld)
				FldSetInsPtPosition(fld, 0);
			break;
		
		case EditOptionsGotobottomofpage:	// Go to the bottom of the field.
			fld = GetFocusObjectPtr();
			if (fld)
				FldSetInsPtPosition(fld, FldGetTextLength(fld));
			break;
		
		case EditEditCut:			// Do a cut to the clipboard if text is highlighted.
			fld = GetFocusObjectPtr();
			if (fld)
				FldCut(fld);
			break;
		
		case EditEditCopy:			// Copy the highlighted text to the clipboard.
			fld = GetFocusObjectPtr();
			if (fld)
				FldCopy(fld);	
			break;
		
		case EditEditPaste:			// Do a paste from the clipboard.
			fld = GetFocusObjectPtr();
			if (fld)
				FldPaste(fld);		
			break;
			
		case EditEditUndo:			// Undo the last text change.
			fld = GetFocusObjectPtr();
			if (fld)
				FldUndo(fld);
			break;
		
		case EditEditSelectAll:	// Select all the existing text.
			fld = GetFocusObjectPtr();
			if (fld)
				FldSetSelection(fld, 0, FldGetTextLength(fld));
			break;
			
		case EditEditKeyboard:		// Display the on screen keyboard.
			SysKeyboardDialog(kbdAlpha);
			break;

		case EditEditGraffiti:		// Display the graffiti reference screen
			SysGraffitiReferenceDialog (referenceDefault);
			break;			
		}	
}


/***********************************************************************
 *
 * FUNCTION:		EditViewHandleEvent
 *
 * DESCRIPTION:	P3. Handles processing of events for the edit form.
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean EditViewHandleEvent(EventPtr event)
{
	FormPtr	frm;
	Word		fldIndex;
	Boolean	handled = false;

	switch (event->eType)
		{
   	case ctlSelectEvent:  // A control button was pressed and released.
			
	   	// If the done button is pressed, go back to the main form.
	   	if (event->data.ctlEnter.controlID == MemoPadEditDoneButton)
	  		 	{
				// Remove the edit form and display the main form.
	   		FrmGotoForm(MemoPadMainForm);
				handled = true;
				}

	   	// P10. If the details button is pressed, pop up the details form.
			else if (event->data.ctlEnter.controlID == MemoPadEditDetailsButton)
				{
				FrmPopupForm(MemoPadDetailsForm);
				handled = true;
				}

	   	// P11. If the category trigger is pressed, pop up the category list.
			else if (event->data.ctlEnter.controlID == MemoPadEditCategoryPopTrigger)
				{
				EditViewSelectCategory ();
				handled = true;
				}
			break;
  	

  		case frmUpdateEvent:	// P11. The form must update it's presentation
			handled =  EditViewUpdateDisplay (event->data.frmUpdate.updateCode);
			break;


  		case frmOpenEvent:	// The form was told to open.
  			// It has already been loaded and activated.
  			
  			// P5. Get a pointer to the form and the index of the text field.
			frm = FrmGetActiveForm();
			fldIndex = FrmGetObjectIndex(frm, MemoPadEditEditFieldField);

   		// P5. Get the data for the field from the application's database.
   		EditViewRetrieveData(FrmGetObjectPtr(frm, fldIndex));
			
  			// Draw the form with the text field data in place.
			FrmDrawForm(frm);
			
			// P4. Set the focus, and the insertion point, to the text edit field.
			FrmSetFocus(frm, fldIndex);
			handled = true;
			break;
			
		case menuEvent:		// P4. A menu item was selected.
			// First clear the menu status from the display.
			MenuEraseStatus(0);
			
			// Process menu commands for the edit form.
			EditViewDoMenuCommand(event->data.menu.itemID);
			handled = true;
			break;

  		case frmCloseEvent:
  			// P5. Get a pointer to the form and the index of the text field.
   		frm = FrmGetActiveForm();
   		fldIndex = FrmGetObjectIndex(frm, MemoPadEditEditFieldField);
   		
   		// P5. Save the data from the field to the application's database.
   		EditViewSaveData(FrmGetObjectPtr(frm, fldIndex));
			break;
		}
	return(handled);
}


/***********************************************************************
 *
 * FUNCTION:		MainViewLoadRecords
 *
 * DESCRIPTION:	P11. Loads the list object.
 *
 * PARAMETERS:		form - The pointer to the form containing the list object.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void MainViewLoadRecords(FormPtr frm)
{
	ListPtr			lst;
	UInt				numRecords;


	// P11. Get the number of records currently in the application's database.
	// DmNumRecordsInCategory doesn't count hidden records.
	numRecords = DmNumRecordsInCategory (MemoPadDB, CurrentCategory);

	// Prepare to setup the contents of the list for the main form.
	
	// Get a pointer to the list object.
	lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MemoPadMainMemolistList));
	
	// P8. Set the number of list choices.
	LstSetListChoices(lst, NULL, numRecords);

	// P8. Set the callback function to draw each list entry.
	LstSetDrawFunction(lst, MainViewListDrawItem);

}


/***********************************************************************
 *
 * FUNCTION:    MainViewSelectCategory
 *
 * DESCRIPTION: P11. This routine handles selection, creation and deletion of
 *              categories for the Main View.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    The index of the new category.
 *
 *              The following global variables are modified:
 *							CurrentCategory
 *							CategoryName
 *
 ***********************************************************************/
static Word MainViewSelectCategory (void)
{
	FormPtr frm;
	ListPtr listP;
	Word category;
	Boolean categoryEdited;

	
	// Process the category popup list.  
	category = CurrentCategory;

	frm = FrmGetActiveForm();
	categoryEdited = CategorySelect (MemoPadDB, frm, MemoPadMainCategoryPopTrigger,
					  MemoPadMainCategoriesListList, true, &category, CategoryName,1,0);
	
	if (categoryEdited || (category != CurrentCategory))
		{
		CurrentCategory = category;

		// Display the new category.
		listP = GetObjectPtr (MemoPadMainMemolistList);
		MainViewLoadRecords (frm);
		LstDrawList (listP);
		}

	return (category);
}


/***********************************************************************
 *
 * FUNCTION:		MainViewListDrawItem
 *
 * DESCRIPTION:	P8. Draw an item in the main form's list.  This routine is 
 *						called from LstDrawList once for each item in the list.
 *
 * PARAMETERS:		itemNum - which list item to draw
 *						bounds - bounds in which to draw the item
 *						itemsText - pointer to data (not used)
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void MainViewListDrawItem(UInt itemNum, RectanglePtr bounds, CharPtr *itemsText)
{
	Handle		recHandle;
	CharPtr		recText;
	UInt			x;
	Int			textLen, lstWidth;
	Boolean		fits;
	UInt			recordNum = 0;
	Err			error;
	
	
	// P11. Find the record to draw, skipping hidden records.
	error = DmSeekRecordInCategory (MemoPadDB, &recordNum, itemNum,
		dmSeekForward, CurrentCategory);
	ErrFatalDisplayIf(error, "Could not find a record to draw.");
	
	// Retrieve the record from the database and lock it down.
	recHandle = DmGetRecord(MemoPadDB, recordNum);
	recText = MemHandleLock(recHandle);
	
	// Determine the length of text that will fit within the list bounds.
	lstWidth = bounds->extent.x - 2;
	textLen = StrLen(recText);
	FntCharsInWidth(recText, &lstWidth, &textLen, &fits);
	
	// Now draw the text from the record.
	x = bounds->topLeft.x;
	WinDrawChars(recText, textLen, x, bounds->topLeft.y);
	
	// Unlock the handle to the record.
	MemHandleUnlock(recHandle);
	
	// Release the record, not dirty.
	DmReleaseRecord(MemoPadDB, recordNum, false);
}


/***********************************************************************
 *
 * FUNCTION:		MainViewInit
 *
 * DESCRIPTION:	P7. Initializes the main form and the list object.
 *
 * PARAMETERS:		frm - pointer to the main form.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void MainViewInit(void)
{
	FormPtr			frm;


	// Get a pointer to the main form.
	frm = FrmGetActiveForm();
	
	MainViewLoadRecords(frm);

	// P11. Set the label of the category trigger.
	CategoryGetName (MemoPadDB, CurrentCategory, CategoryName);
	CategorySetTriggerLabel (GetObjectPtr (MemoPadMainCategoryPopTrigger), CategoryName);

	// Draw the form.
	FrmDrawForm(frm);
}


/***********************************************************************
 *
 * FUNCTION:		MainViewHandleEvent
 *
 * DESCRIPTION:	Handles processing of events for the main form.
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean MainViewHandleEvent(EventPtr event)
{
	Boolean		handled = false;
	FormPtr		frm;
	ListPtr		lst;

	switch (event->eType)
		{
   	case ctlSelectEvent:  // A control button was pressed and released.
	   	if (event->data.ctlEnter.controlID == MemoPadMainNewButton)
	   		{
				// P5. Flag to the edit form that a new record is needed.
				// CurrentRecord = noRecordSelected;
				
				// P6. Create a new database record.
				if (CreateRecord())
					{
					// P3. The new button is pressed, go to the edit form.
					FrmGotoForm(MemoPadEditForm);
					}
				handled = true;
				}
			
	   	// P11. If the category trigger is pressed, pop up the category list.
	   	else if (event->data.ctlEnter.controlID == MemoPadMainCategoryPopTrigger)
	   		{
				MainViewSelectCategory();
				handled = true;
				}
			break;
  	
  		case lstSelectEvent:  // P7. An entry in the list was selected.
			// Get the current selection from the list and set the current record to edit.
			frm = FrmGetActiveForm();
			lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MemoPadMainMemolistList));
			CurrentRecord = 0;
			DmSeekRecordInCategory(MemoPadDB, &CurrentRecord, LstGetSelection(lst),
				dmSeekForward, CurrentCategory);
			
			// Go to the edit form.	
			FrmGotoForm(MemoPadEditForm);
			handled = true;
			break;

  		case frmOpenEvent:	// P3. The form was told to open.
  			// P7. Initialize the main form.
			MainViewInit();
			handled = true;
			break;
			
		case frmCloseEvent:	// P7. The form was told to close.
			// Free the memory blocks allocated for the list.
			if (ChoicesHandle)
				{
				MemHandleFree(ChoicesHandle);
				ChoicesHandle = 0;
				MemHandleFree(ChoicesPtrsHandle);
				}
			break;
	
		case menuEvent:		// P2. A menu item was selected.
			// The get info being the only menu item, display the info form.
			
			// First clear the menu status from the display.
			MenuEraseStatus(0);
			
			// Load the info form, then display it.
			frm = FrmInitForm(MemoPadInfoForm);
			FrmDoDialog(frm);
			
			// Delete the info form.
	 		FrmDeleteForm(frm);
			handled = true;
			break;
		}
	return(handled);
}


/***********************************************************************
 *
 * FUNCTION:    ApplicationHandleEvent
 *
 * DESCRIPTION: P3. This routine loads a form resource and sets the event handler for the form.
 *
 * PARAMETERS:  event - a pointer to an EventType structure
 *
 * RETURNED:    True if the event has been handled and should not be
 *						passed to a higher level handler.
 *
 ***********************************************************************/
static Boolean ApplicationHandleEvent(EventPtr event)
{
	FormPtr	frm;
	Word		formId;
	Boolean	handled = false;

	if (event->eType == frmLoadEvent)
		{
		// Load the form resource specified in the event then activate the form.
		formId = event->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);

		// Set the event handler for the form.  The handler of the currently 
		// active form is called by FrmDispatchEvent each time it receives an event.
		switch (formId)
			{
			case MemoPadMainForm:
				FrmSetEventHandler(frm, MainViewHandleEvent);
				break;
		
			case MemoPadEditForm:
				FrmSetEventHandler(frm, EditViewHandleEvent);
				break;
		
			case MemoPadDetailsForm:		// P10.
				FrmSetEventHandler(frm, DetailsFormHandleEvent);
				break;
			}
		handled = true;
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:		EventLoop
 *
 * DESCRIPTION:	A simple loop that obtains events from the Event
 *						Manager and passes them on to various applications and
 *						system event handlers before passing them on to
 *						FrmDispatchEvent for default processing.
 *
 * PARAMETERS:		None.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void EventLoop(void)
{
	EventType	event;
	Word			error;
	
	do
		{
		// Get the next available event.
		EvtGetEvent(&event, evtWaitForever);
		
		// Give the system a chance to handle the event.
		if (! SysHandleEvent(&event))

			// P2. Give the menu bar a chance to update and handle the event.	
			if (! MenuHandleEvent(0, &event, &error))

				// P3. Give the application a chance to handle the event.
				if (! ApplicationHandleEvent(&event))

					// P3. Let the form object provide default handling of the event.
					FrmDispatchEvent(&event);
		}
	while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:		PilotMain
 *
 * DESCRIPTION:	This function is the equivalent of a main() function
 *						under standard C.  It is called by the Emulator to begin
 *						execution of this application.
 *
 * PARAMETERS:		cmd - command specifying how to launch the application.
 *						cmdPBP - parameter block for the command.
 *						launchFlags - flags used to configure the launch.			
 *
 * RETURNED:		Any applicable error codes.
 *
 ***********************************************************************/
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	Word error;		// P4. error starting the app
	
	// P4. This app makes use of PalmOS 2.0 features.  It will crash if
	// run on an earlier version of PalmOS.  Detect and warn if this happens,
	// then exit.
	error = RomVersionCompatible (version20, launchFlags);
	if (error)
		return error;

	// P4. This app doesn't support any action codes.
	if (cmd != sysAppLaunchCmdNormalLaunch) return sysErrParamErr;
	
	// Initialize the application's global variables and database.
	StartApplication();

	// P3. Start the first form.
	FrmGotoForm(CurrentView);
		
	// Start the event loop.
	EventLoop();
	
	// P5. Clean up before exiting the applcation.
	StopApplication();
	
	return 0;
}
