/***********************************************************************
 *
 Copyright  1995 - 1998, 3Com Corporation or its subsidiaries ("3Com").  
 All rights reserved.
   
 This software may be copied and used solely for developing products for 
 the Palm Computing platform and for archival and backup purposes.  Except 
 for the foregoing, no part of this software may be reproduced or transmitted 
 in any form or by any means or used to make any derivative work (such as 
 translation, transformation or adaptation) without express written consent 
 from 3Com.

 3Com reserves the right to revise this software and to make changes in content 
 from time to time without obligation on the part of 3Com to provide notification 
 of such revision or changes.  
 3COM MAKES NO REPRESENTATIONS OR WARRANTIES THAT THE SOFTWARE IS FREE OF ERRORS 
 OR THAT THE SOFTWARE IS SUITABLE FOR YOUR USE.  THE SOFTWARE IS PROVIDED ON AN 
 "AS IS" BASIS.  3COM MAKES NO WARRANTIES, TERMS OR CONDITIONS, EXPRESS OR IMPLIED, 
 EITHER IN FACT OR BY OPERATION OF LAW, STATUTORY OR OTHERWISE, INCLUDING WARRANTIES, 
 TERMS, OR CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND 
 SATISFACTORY QUALITY.

 TO THE FULL EXTENT ALLOWED BY LAW, 3COM ALSO EXCLUDES FOR ITSELF AND ITS SUPPLIERS 
 ANY LIABILITY, WHETHER BASED IN CONTRACT OR TORT (INCLUDING NEGLIGENCE), FOR 
 DIRECT, INCIDENTAL, CONSEQUENTIAL, INDIRECT, SPECIAL, OR PUNITIVE DAMAGES OF 
 ANY KIND, OR FOR LOSS OF REVENUE OR PROFITS, LOSS OF BUSINESS, LOSS OF INFORMATION 
 OR DATA, OR OTHER FINANCIAL LOSS ARISING OUT OF OR IN CONNECTION WITH THIS SOFTWARE, 
 EVEN IF 3COM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

 3Com, HotSync, Palm Computing, and Graffiti are registered trademarks, and 
 Palm III and Palm OS are trademarks of 3Com Corporation or its subsidiaries.

 IF THIS SOFTWARE IS PROVIDED ON A COMPACT DISK, THE OTHER SOFTWARE AND 
 DOCUMENTATION ON THE COMPACT DISK ARE SUBJECT TO THE LICENSE AGREEMENT 
 ACCOMPANYING THE COMPACT DISK.

 ********************************************************************************
 *
 * PROJECT:			Memo Pad Sample Application
 *
 * FILE:				MemoPad.c (Phase 17)
 *
 *		For best formatting set tab size to 3.
 *
 * DESCRIPTION:	Sample application working with the desktop.
 *	  
 * OBJECTIVE:		Use DmDeleteRecord instead of DmRemoveRecord.
 *						Handle sysAppLaunchCmdSyncNotify.
 *
 * REVISION HISTORY:
 * 	 1/17/96	Roger		Initial version
 *
 **********************************************************************/
 
#include <Pilot.h>				// all the system toolbox headers
#include <SysEvtMgr.h>			//	P14. Needed for search for EvtSysEventAvail
#include <FeatureMgr.h>			//	P4. Needed to get the ROM version
#include "MemoPadRsc.h"			// application resource defines


/***********************************************************************
 * Structures for this module
 **********************************************************************/

// P11. An application info structure is intended to be stored in the
// database's app info block and to contain data relevant to the database.
// For now we store just category information.  Category information is 
// always placed in the beginning where the category.c code expects it.
typedef struct 
	{
   UInt	renamedCategories;		// bitfield of categories with a changed name
	char	categoryLabels[dmRecNumCategories][dmCategoryLength];
	Byte	categoryUniqIDs[dmRecNumCategories];
	Byte	lastUniqID;			// Uniq IDs for categories generated by the device 
									// are between 0 - 127.  Those from the PC are 128 - 255.
	Byte	reserved1;			// from the compiler word aligning things
	Word	reserved2;
	} MemoPadAppInfoType;

typedef MemoPadAppInfoType * MemoPadAppInfoPtr;		// P11.
typedef CharPtr MemoPadDBRecordTypePtr;				// P14.

// P15. The data saved between application executions.  This data
// is saved as a single resource in the System Preferences.
typedef struct 
	{
	UInt				currentCategory;
	UInt				topVisibleRecord;
	UInt				currentRecord;
	ULong				currentRecordID;		// P17. we need this incase a synchronization
													// moves the record.
	} MemoPadPreferenceType;


/***********************************************************************
 * Global defines for this module
 **********************************************************************/
#define version20	0x02000000	// P4. PalmOS 2.0 version number

// Application database management defines.
#define memoPadAppType		'Memo'			// P5. type for application.  must be 4 chars, mixed case.
#define memoPadDBType		'Data'			// P5. type for application database.  must be 4 chars, mixed case.
#define memoPadDBName		"MemoPadDB"		// P5. name for application database.  up to 31 characters.
#define memoPadPrefID		0					// preferences resource ID.
#define memoPadVersionNum	1					// P15. version of the software
#define noRecordSelected	-1					// P5. index for no current record.
#define noRecordSelectedID	-1					// P15. id for no unique id
#define newMemoSize  		64					// P6. initial size for new database record.
#define memosInCategoryUnknown	0xffff	// P16. recalculate MemosInCategory before using

#define updateCategoryChanged		0x01		// P11. Indicates a category changed
#define localizedAppInfoStr		1000		// P11. default category strings ** tAIS **

#define titleReplacementChar		'#'	// P16. Character replaced in edit title
#define digitsForRecordPosition	6		// P16. digits in title replacements
#define editFontGroup				1		// P16. group number of font pushbuttons
 

/***********************************************************************
 * Global variables for this module
 **********************************************************************/
static DmOpenRef		MemoPadDB;				// P5. handle for database
static char				CategoryName [dmCategoryLength];	// P11. Storage for the name
static Word				RecordCategory;		// P11. edited record's category (never All)
static Word 			NewCategory;			// P11. category in details form (never All)
static Boolean			NewCategoryEdited;	// P11. true if name edited
static CharPtr			EditViewTitlePtr = 0;		// P16. allocated edit title
static Word				MemosInCategory;				// P16. records in one or all categories
static FontID			EditViewFont = stdFont;		// P16. font used by field in edit view

// P11. This is a default MemoPadAppInfoType structure.  When a new database
// is created and a default app info block is created, the following structure
// is copied into the block initializing it.
static MemoPadAppInfoType DefaultMemoPadAppInfo = 
	{
 	// Renamed categories
 	0x0006,			// set the 2nd and 3rd bits indicating those categories as changed.

	"Unfiled",
	"Business",
	"Personal",
	"",
	"",	"",	"",	"",
	"",	"",	"",	"",
	"",	"",	"",	"",
	
	// Category Uniq IDs
	0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 
	
	// Last Uniq ID
	15,

	// Reserved
	0, 0
	};


// P15. The following global variable are saved to a state file.
static Int				CurrentView;			// P3. id of current form
static UInt				CurrentRecord = noRecordSelected;	// P5. index of current database record
static Int				CurrentCategory = dmAllCategories;	// P11. category filter
static Int				TopVisibleRecord = 0;// P12. top record in main table



/***********************************************************************
 * Prototypes for internal functions
 **********************************************************************/
static Boolean StartApplication(void);
static void StopApplication(void);								// P5. clean up before app exit
static VoidPtr GetObjectPtr(Int objectID);					// P10. get ptr to object in active form
static FieldPtr GetFocusObjectPtr(void);						// P4. get ptr to text field
static Boolean CreateRecord(void);								// P6. create a new record in the application database
static void DetailsInit(void);									// P10. initialize data, objects for the details form
static UInt DetailsApply(void);									// P10. apply changes made in the details form
static Boolean DetailsFormHandleEvent(EventPtr event);	// P10. handle events for the details form
static void EditViewDoMenuCommand(Int command);					// P4. perform commands for edit form menu
static Boolean EditViewDeleteCurrentRecord(void);				// P10. delete database record
static void EditViewSaveData(FieldPtr fld);						// P5. save data to application database
static void EditViewRetrieveData(FieldPtr fld);					// P5. get data from application database
static Boolean EditViewHandleEvent(EventPtr event);
static void MainViewDrawRecordInBounds(						// P14. Draw a record
	MemoPadDBRecordTypePtr memoPadRecP, RectanglePtr bounds);
static void MainViewInit(void);									// P7. initialize data, objects for the main form
static Boolean MainViewHandleEvent(EventPtr event);
static Boolean ApplicationHandleEvent(EventPtr event);	// P3. handle form load events
static void EventLoop(void);


/************************************************************
 *
 *  FUNCTION: MemoPadInitAppInfo
 *
 *  DESCRIPTION: P11. Create an app info chunk if missing.  Set
 *		the category strings to a default.
 *
 *  PARAMETERS: database pointer
 *
 *  RETURNS: 0 if successful, errorcode if not
 *
 *************************************************************/
static Err	MemoPadInitAppInfo(DmOpenRef dbP)
{
	UInt cardNo;
	VoidHand h;
	LocalID dbID;
	LocalID appInfoID;
	MemoPadAppInfoPtr	appInfoP;
	MemoPadAppInfoPtr	nilP = 0;

	
	// We have a DmOpenRef and we want the database's app info block ID.
	// Get the database's dbID and cardNo and then use them to get the appInfoID.
	if (DmOpenDatabaseInfo(dbP, &dbID, NULL, NULL, &cardNo, NULL))
		return dmErrInvalidParam;
	if (DmDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, 
			NULL, NULL, &appInfoID, NULL, NULL, NULL))
		return dmErrInvalidParam;


	// If no appInfoID exists then we must create a new one.	
	if (appInfoID == NULL)
		{
		h = DmNewHandle (dbP, sizeof (MemoPadAppInfoType));
		if (! h) return dmErrMemError;
		
		appInfoID = MemHandleToLocalID (h);
		DmSetDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &appInfoID, NULL, NULL, NULL);
		}


	// Lock the appInfoID and copy in defaults from our default structure.	
	appInfoP = MemLocalIDToLockedPtr(appInfoID, cardNo);

	// Clear the app info block.
	DmSet (appInfoP, 0, sizeof (MemoPadAppInfoType), 0);

	// Initialize the categories.
	CategoryInitialize ((AppInfoPtr) appInfoP, localizedAppInfoStr);

	// Initialize reserved2.  This is actually redundant since the DmSet
	// call above has already set reserved2 to zero.  But this shows how
	// to set variables in addition to the categories.
	DmSet (appInfoP, (ULong)&nilP->reserved2, sizeof(appInfoP->reserved2), 
		0);

	MemPtrUnlock(appInfoP);
	return 0;
}


/***********************************************************************
 *
 * FUNCTION:     StartApplication
 *
 * DESCRIPTION:  This routine sets up the initial state of the application.
 *               It opens the application's database and sets up global variables. 
 *
 * PARAMETERS:   None.
 *
 * RETURNED:     true if error (database couldn't be created)
 *
 ***********************************************************************/
static Boolean StartApplication(void)
{
	Int		error;			// P5. error code
	UInt		cardNo;			// P5. card containing the application database
	LocalID	dbID;				// P5. handle for application database
	UInt		dbAttrs;			// P5. database attributes
	UInt		mode;				// P5. permissions when opening the database
	SystemPreferencesType sysPrefs;	// P10. The system's preferences
	MemoPadPreferenceType prefs;	// P15. app state before closed last time
	Boolean recordFound = false;	// P15. whether we use the record used before
											// This will be false if the record is now hidden.
	LocalID appInfoID;		// P15.
	UInt attr;					// P15. record attributes
	ULong uniqueID;			// P15. record's unique id
	UInt version;				// P17. The database's version.
	Word	prefSize;

	// P10. Determime if secret record should be shown.
	if ( PrefGetPreference(prefHidePrivateRecords) )
		mode = dmModeReadWrite;
	else
		mode = dmModeReadWrite | dmModeShowSecret;

	// P5. Find the application's database.
	MemoPadDB = DmOpenDatabaseByTypeCreator(memoPadDBType, memoPadAppType, mode);
	if (! MemoPadDB)
		{
		// P5. The database doesn't exist, create it now.
		error = DmCreateDatabase(0, memoPadDBName, memoPadAppType, memoPadDBType, false);

		// P5. Check for error.  If the database couldn't be created report it.
		// The application should exit if the database couldn't be created.
		if (error)
			return true;
		
		// P5. Find the application's database.
		MemoPadDB = DmOpenDatabaseByTypeCreator(memoPadDBType, memoPadAppType, mode);

		// P5. Get info about the database
		error = DmOpenDatabaseInfo(MemoPadDB, &dbID, NULL, NULL, &cardNo, NULL);

		// P5. Check for fatal error.
		ErrFatalDisplayIf(error, "Could not get database info.");
	
		// P5. Get attributes for the database
		error = DmDatabaseInfo(0, dbID, NULL, &dbAttrs, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

		// P5. Check for fatal error.
		ErrFatalDisplayIf(error, "Could not get database info.");
	
		// P17. Set the backup flag in the attributes
		// This will ensure that the database will always get backed up
		// during HotSync, even if there is no conduit for this application.
		dbAttrs = dbAttrs | dmHdrAttrBackup;

		// P5. Set the new attributes in the database
		error = DmSetDatabaseInfo(0, dbID, NULL, &dbAttrs, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

		// P5. Check for fatal error.
		ErrFatalDisplayIf(error, "Could not set database info.");
		
		// P11. Store category info in the application's information block.
		// If error return so.
		if (MemoPadInitAppInfo(MemoPadDB))
			return true;
		}
	else
		// P17. After a hard reset, the records may be sent to the device.
		// In this case the database might not have an app info block.
		// Check for an app info block, and if there isn't one create a
		// default one.
		{
		// P17. Get info about the database
		error = DmOpenDatabaseInfo(MemoPadDB, &dbID, NULL, NULL, &cardNo, NULL);

		// P17. Check for fatal error.
		ErrFatalDisplayIf(error, "Could not get database info.");
	
		// P17. Get attributes for the database.  This is an apropriate place
		// to get the version number of the database for applications which
		// have multiple versions of databases.  If the application detects
		// a different version of the database it may decide to not use it,
		// to translate it, or not support some new or old features.
		error = DmDatabaseInfo(0, dbID, NULL, NULL, &version, NULL, NULL, NULL, 
			NULL, &appInfoID, NULL, NULL, NULL);

		// P17. Check for fatal error.
		ErrFatalDisplayIf(error, "Could not get database info.");

		if (appInfoID == NULL)
			{
			// P17. Store category info in the application's information block.
			if (MemoPadInitAppInfo(MemoPadDB))
				return true;
			}
		}
	

	// P15. Read the preferences / saved-state information.  If the preferences
	// were written with a different memoPadVersionNum they will be ignored.  If
	// MemoPadPreferenceType changes then memoPadVersionNum should be incremented
	// to avoid using outdated versions of the preferences.
	PrefGetPreferences(&sysPrefs);
	prefSize = sizeof(prefs);
	if (PrefGetAppPreferences (memoPadAppType, memoPadPrefID, &prefs, 
		&prefSize, true) == memoPadVersionNum)
		{
		TopVisibleRecord = prefs.topVisibleRecord;
		CurrentRecord = prefs.currentRecord;
		CurrentCategory = prefs.currentCategory;
		}
		

	// P11. Get the name of the current category from the app info block.
	// The app info block was created and initialized by MemoPadInitAppInfo.
	CategoryGetName (MemoPadDB, CurrentCategory, CategoryName);

	// P17. The category may no longer exist if it was removed during a sync
	if (*CategoryName == 0)
		{
		CurrentCategory = dmAllCategories;
		}

	
	// P17. Because synchronization can move the records around the 
	// way the old record is found needs careful thought.  In prior
	// phases we simply looked up the record by it's record number.
	// A synchronization makes that record number unreliable.  We
	// can look up the record by it's unique ID.  This works well but 
	// is much slower.  A hybrid which looks up the record at record 
	// number and then if the unique ID doesn't match look up the record
	// by unique ID presents the advantages of the two other schemes.
	// Usually it is quick like the original method.  But in the event
	// of a unique ID mismatch the hybrid yields an accurate find like
	// the second method.  The hybrid is what we use.
	
	// P15. Use the same record as before.  We should, however, make sure
	// that the record isn't private now or may be moved or deleted (by
	// a sync).
	if (CurrentRecord != noRecordSelected &&
		 CurrentRecord < DmNumRecords(MemoPadDB))
		{
		DmRecordInfo (MemoPadDB, CurrentRecord, &attr, &uniqueID, NULL);
		}
	else
		{
		uniqueID = prefs.currentRecordID - 1;	// force not equal
		}
	
	// P17. If we don't have the record find it by it's unique id.  
	// Then get it's info.
	if (CurrentRecord != noRecordSelected &&
		uniqueID != prefs.currentRecordID)
		{
		if (!DmFindRecordByID (MemoPadDB, prefs.currentRecordID, &CurrentRecord))
			DmRecordInfo (MemoPadDB, CurrentRecord, &attr, &uniqueID, NULL);
		}

	// P15. Now check the private settings to make sure we can display it.
	// P17. Also make sure the record is the same one saved.
	recordFound = (uniqueID == prefs.currentRecordID) &&
					  ((!sysPrefs.hideSecretRecords) || (!(attr & dmRecAttrSecret)));

	// P15. If we can't show the record we used to have then just start normally.
	if (! recordFound)
		{
		TopVisibleRecord = 0;
		CurrentRecord = noRecordSelected;
		}
	
	// P16. Count the records later when needed to display.
	MemosInCategory = memosInCategoryUnknown;
	
	
	// P4. Set current view so the first form to display is the main form.
	CurrentView = MemoPadMainForm;
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: P4. Check that the ROM version meets your
 *              minimum requirement.  Warn if the app was switched to.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags indicating how the application was
 *											 launched.  A warning is displayed only if
 *                                these flags indicate that the app is 
 *											 launched normally.
 *
 * RETURNED:    zero if rom is compatible else an error code
 *                             
 ***********************************************************************/
static Err RomVersionCompatible (DWord requiredVersion, Word launchFlags)
{
	DWord romVersion;
	
	
	// See if we're on in minimum required version of the ROM or later.
	// The system records the version number in a feature.  A feature is a
	// piece of information which can be looked up by a creator and feature
	// number.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		// If the user launched the app from the launcher, explain
		// why the app shouldn't run.  If the app was contacted for something
		// else, like it was asked to find a string by the system find, then
		// don't bother the user with a warning dialog.  These flags tell how
		// the app was launched to decided if a warning should be displayed.
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Pilot 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.  The sysFileCDefaultApp is considered "safe".
			if (romVersion < 0x02000000)
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		
		return (sysErrRomIncompatible);
		}

	return 0;
}

/***********************************************************************
 *
 * FUNCTION:    StopApplication
 *
 * DESCRIPTION: P15. This routine closes the application's database
 *              and saves the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void StopApplication(void)
{
	MemoPadPreferenceType prefs;
	

	// P11. Close all open forms to allow their frmCloseEvent handlers
	// to execute.  An appStopEvent doesn't send frmCloseEvents.
	// FrmCloseAllForms will send all opened forms a frmCloseEvent.
	FrmCloseAllForms ();


	// Write the preferences / saved-state information.
	prefs.topVisibleRecord = TopVisibleRecord;
	prefs.currentRecord = CurrentRecord;
	prefs.currentCategory = CurrentCategory;
		
	// Get the current record's unique id and save it with the state 
	// information.
	if ( DmQueryRecord (MemoPadDB, CurrentRecord) != 0)
		{
		DmRecordInfo (MemoPadDB, CurrentRecord, NULL, &prefs.currentRecordID, NULL);
		}
	else
		prefs.currentRecordID = noRecordSelectedID;

	// Write the state information.
	PrefSetAppPreferences (memoPadAppType, memoPadPrefID, memoPadVersionNum, &prefs, 
		sizeof (MemoPadPreferenceType), true);

	// Close the application's database.
	DmCloseDatabase(MemoPadDB);
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: P10. This routine returns a pointer to an object in the active form.
 *
 * PARAMETERS:  objectID - id of the object
 *
 * RETURNED:    pointer to the object's data structure
 *
 ***********************************************************************/
static VoidPtr GetObjectPtr(Int objectID)
{
	FormPtr frm;
	
	frm = FrmGetActiveForm();
	return(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectID)));
}

/***********************************************************************
 *
 * FUNCTION:    GetFocusObjectPtr
 *
 * DESCRIPTION: P4. This routine returns a pointer to the field object, in 
 *              the current form, that has the focus.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    pointer to a field object or NULL of there is no focus
 *
 ***********************************************************************/
static FieldPtr GetFocusObjectPtr(void)
{
	FormPtr frm;
	Int focus;
	
	// Get a pointer to tha active form and the index of the form object with focus.
	frm = FrmGetActiveForm();
	focus = FrmGetFocus(frm);
	
	// If no object has the focus return NULL pointer.
	if (focus == noFocus)
		return(NULL);
		
	// Return a pointer to the object with focus.
	return(FrmGetObjectPtr(frm, focus));
}


/***********************************************************************
 *
 * FUNCTION:    ChangeCategory
 *
 * DESCRIPTION: P11. Updates the edit view and it's variables.  
 *
 * PARAMETERS:  category  - new category (index)
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void ChangeCategory (Word category)
{
	// Remember what category the record is in.
	RecordCategory = category;
	
	// P16. Count the records later when needed to display.
	MemosInCategory = memosInCategoryUnknown;

	// The MemoPadEditForm should be updated to display the new category.
	// Send a frmUpdateEvent with our updateCategoryChanged code
	// which we defined. 
	FrmUpdateForm (MemoPadEditForm, updateCategoryChanged);
}


/***********************************************************************
 *
 * FUNCTION:    Search
 *
 * DESCRIPTION: P14. Search the database for records 
 *              containing the string passed.
 *
 *					 This routine is called in response to a system launch 
 * 				 code from PilotMain.   The application may not
 *					 be running and if so the applications global variables
 *					 are not set up.  This means the global variables may not
 *					 be read and may not be written to.
 *
 *					 Because this routine is called whenever the system find
 *					 feature is used this routine must be fast.  Slow performance
 *					 makes the whole device seem slow.
 *
 * PARAMETERS:	 findParams - text search parameter block setup by the system
 *
 * RETURNED:	 nothing
 *
 ***********************************************************************/
static void Search (FindParamsPtr findParams)
{
	Word pos;
	CharPtr header;
	UInt recordNum;
	VoidHand recordH;
	VoidHand headerH;
	RectangleType r;
	Boolean done;
	Boolean match;
	DmOpenRef dbP;
	UInt cardNo = 0;
	LocalID dbID;
	MemoPadDBRecordTypePtr memoPadRecP;


	// Find the application's data file.
	dbP = DmOpenDatabaseByTypeCreator(memoPadDBType, memoPadAppType, findParams->dbAccesMode);
	if (!dbP)
		{
		// Return without anything.  Also indicate that no more records are expected.
		findParams->more = false;
		return;
		}
	DmOpenDatabaseInfo(dbP, &dbID, 0, 0, &cardNo, 0);

	// Display the heading line.
	headerH = DmGetResource (strRsc, FindHeaderTitleString);
	header = MemHandleLock (headerH);
	done = FindDrawHeader (findParams, header);
	MemHandleUnlock (headerH);
	if (done) goto Exit;				// There was no more room to display the heading line
	

	// Search the records for the string.  We start with the recordNum passed.
	// This allows the search code to be called multiple times when more than
	// one screen of records are found.
	recordNum = findParams->recordNum;
	while (true)
		{
		// Because applications can take a long time to finish a find when
		// the result may be on the screen or for other reasons, users like
		// to be able to stop the find.  Stop the find if an event is pending.
		// This stops if the user does something with the device.  Because
		// this call slows down the search we perform it every so many 
		// records instead of every record.  The response time should still
		// be short without introducing much extra work to the search.
		
		// Note that in the implementation below, if the next 16th record is  
		// secret the check doesn't happen.  Generally this shouldn't be a 
		// problem since if most of the records are secret then the search 
		// won't take long anyways!  Every 16 records is guessed at to check 
		// often enough to respond quickly to the user without wasting time
		// checking too often.
		if ((recordNum & 0x000f) == 0 &&			// every 16th record
			EvtSysEventAvail(true))
			{
			// Stop the search process.
			findParams->more = true;
			break;
			}
		
		// Get the next record.  Skip private records if neccessary.
		recordH = DmQueryNextInCategory (dbP, &recordNum, dmAllCategories);

		// Have we run out of records?
		if (! recordH)
			{
			findParams->more = false;			
			break;
			}

		memoPadRecP = MemHandleLock (recordH);
		 
		// Search for the string passed,  if it's found display the title
		// of the memo.  An application should call FindStrInStr
		// for each part of the record which needs to be searched.
		match = FindStrInStr (memoPadRecP, findParams->strToFind, &pos);
		if (match)
			{
			// Add the match to the find parameter block,  if there is no room to
			// display the match the following function will return true.
			done = FindSaveMatch (findParams, recordNum, pos, 0, 0, cardNo, dbID);

			if (! done)
				{
				// Get the bounds of the region where we will draw the results.
				FindGetLineBounds (findParams, &r);
				
				// Display the record in the search dialog.  We move in one pixel
				// so that when the record is inverted the left edge is solid.
				r.topLeft.x++;
				r.extent.x--;
				MainViewDrawRecordInBounds(memoPadRecP, &r);
	
				// The line number needs to be increment since a line is used for the record.
				findParams->lineNumber++;
				}
			}
		MemHandleUnlock (recordH);

		if (done) break;
		recordNum++;
		}
		
		
Exit:
	DmCloseDatabase (dbP);	
}


/***********************************************************************
 *
 * FUNCTION:    GoToRecord
 *
 * DESCRIPTION: P14. Cause a record to be displayed.  This routine is
 *              generally called because the "Go to" button in the text 
 *					 search dialog is pressed.
 *
 *					 This routine is called in response to a system launch 
 * 				 code from PilotMain.  PilotMain has made sure that global
 *					 variables exist because the application is intended to 
 *					 to remain running so the user can observe the record.
 *
 *					 The record identified by the parameter block passed should
 *              be displayed with the character range specified highlighted.
 *
 * PARAMETERS:	 goToParams   - parameter block that identifies the record to
 *                             display.
 *              launchingApp - true if the application is being launched.
 *									  - false if the record found exists in the current app
 *
 * RETURNED:	 nothing
 *
 ***********************************************************************/
static void GoToRecord (GoToParamsPtr goToParams, Boolean launchingApp)
{
	Word recordNum;
	UInt attr;
	ULong uniqueID;
	EventType event;


	recordNum = goToParams->recordNum;
	DmRecordInfo (MemoPadDB, recordNum, &attr, &uniqueID, NULL);

	// Change the current category if necessary.
	if (CurrentCategory != dmAllCategories)
		{
		ChangeCategory (attr & dmRecAttrCategoryMask);
		}


	// If the application is already running, close all open forms.  
	// Because closing forms sometimes writes data to a record which
	// could affect the ordering of records in the database, the record
	// is looked up by it's unique id which is unaffected by changing
	// data and database ordering.
	if (! launchingApp)
		{
		FrmCloseAllForms ();
		DmFindRecordByID (MemoPadDB, uniqueID, &recordNum);
		}
		

	// Send events to load and goto a form selecting the matching text.
	MemSet (&event, sizeof(EventType), 0);

	event.eType = frmLoadEvent;
	event.data.frmLoad.formID = MemoPadEditForm;
	EvtAddEventToQueue (&event);
 
	event.eType = frmGotoEvent;
	event.data.frmGoto.recordNum = recordNum;
	event.data.frmGoto.matchPos = goToParams->matchPos;
	event.data.frmGoto.matchLen = goToParams->searchStrLen;
	event.data.frmGoto.matchFieldNum = goToParams->matchFieldNum;
	event.data.frmGoto.formID = MemoPadEditForm;
	EvtAddEventToQueue (&event);
}


/***********************************************************************
 *
 * FUNCTION:    CreateRecord
 *
 * DESCRIPTION: P6. This routine creates a new memo record.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    true if the record was sucessfully created.
 *
 ***********************************************************************/
static Boolean CreateRecord(void)
{
	Ptr		p;
	VoidHand memoRec;
	UInt		index = 0;
	Err		error;
	Char		zero = 0;
	UInt		attr;			// P11. record attributes (category)

	// Create a new first record in the database
	memoRec = DmNewRecord(MemoPadDB, &index, newMemoSize);

	// Lock down the block containing the new record.
	p = MemHandleLock(memoRec);

	// Write a zero to the first byte of the record to null terminate the new memo string.
	error = DmWrite(p, 0, &zero, sizeof(Char));
	
	// Check for fatal error.
	ErrFatalDisplayIf(error, "Could not write to new record.");
	
	// Unlock the block of the new record.
	MemPtrUnlock(p);


	// P11. Set the category of the new record to the current category.
	DmRecordInfo (MemoPadDB, index, &attr, NULL, NULL);
	attr &= ~dmRecAttrCategoryMask;				// Remove all category bits
	if (CurrentCategory == dmAllCategories)
		{
		// Since the user isn't looking in any particular category place
		// the new record in the unfiled category.
		attr |= dmUnfiledCategory;
		}
	else
		{
		// Set the attribute bits to indicate the current category.
		attr |= CurrentCategory;
		}
	DmSetRecordInfo (MemoPadDB, index, &attr, NULL);

	if (MemosInCategory != memosInCategoryUnknown)
		MemosInCategory++;			//   P16. the category has gained one record
	
	// Release the record to the database manager.  The true value indicates that
	// the record contains "dirty" data.  Release Record will set the record's 
	// dirty flag and update the database modification count.
	DmReleaseRecord(MemoPadDB, index, true);

	// Remember the index of the current record.
	CurrentRecord = 0;

	return(true);	
}


/***********************************************************************
 *
 * FUNCTION:    DetailsApply
 *
 * DESCRIPTION: P10. This routine applies the changes made in the 
 *              details form to the current record.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    P11. code which indicates how the memo was changed
 *					 the code will be sent as the update code, in a frmUpdate event
 *
 ***********************************************************************/
static UInt DetailsApply(void)
{
	UInt		attr;
	Boolean	secret;
	UInt		updateCode = 0;
	Boolean	dirty = false;		// P11. If an attribute changed
	
	// Get the secret attribute of the current record.
	DmRecordInfo(MemoPadDB, CurrentRecord, &attr, NULL, NULL);

	// Get the current setting of the secret checkbox.
	secret = (CtlGetValue(GetObjectPtr(MemoPadDetailsPrivateCheckbox)) != 0);

	// Has the secret attribute been changed?
	if (((attr & dmRecAttrSecret) == dmRecAttrSecret) != secret)
		{
		// Yes, it was changed.  What is the new setting?
		if (secret)
			// Now it should be secret.
			attr |= dmRecAttrSecret;
		else
			// Now it should not be secret.
			attr &= ~dmRecAttrSecret;

		dirty = true;
		}


	// P11. Has the category been changed?
	if (RecordCategory != NewCategory)
		{
		attr &= ~dmRecAttrCategoryMask;		// Remove all category bits.
		attr |= NewCategory;						// Set the new category
		dirty = true;								// mark this record dirty later
		RecordCategory = NewCategory;			// remember which category the record is in
		MemosInCategory = memosInCategoryUnknown;	// P16. Count the records later when displayed
		updateCode |= updateCategoryChanged;	// send this update code
		}
	

	// P11. If the current category was deleted, renamed, or merged with
	// another category, then the list view needs to be redrawn.
	if (NewCategoryEdited)
		{
		RecordCategory = NewCategory;
		MemosInCategory = memosInCategoryUnknown;	// P16. Count the records later when displayed
		updateCode |= updateCategoryChanged;
		}


	if (dirty)
		{
		// Set the dirty flag for the record.
		attr |= dmRecAttrDirty;
		DmSetRecordInfo (MemoPadDB, CurrentRecord, &attr, NULL);
		}
	
	return updateCode;
}


/***********************************************************************
 *
 * FUNCTION:    DetailsInit
 *
 * DESCRIPTION: P10. This routine initializes the Details Dialog.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void DetailsInit(void)
{
	ControlPtr	ctl;
	UInt			attr;	
	Boolean 		secret;
	
	
	// P11. NewCategory indicates which category wants to changed the 
	// current record to.  Initialize it to the current record's current
	// category.
	NewCategory = RecordCategory;
	NewCategoryEdited = false;
			
	// Get the secret and category attributes of the current record.
	DmRecordInfo(MemoPadDB, CurrentRecord, &attr, NULL, NULL);
	secret = attr & dmRecAttrSecret;

	// If the record is marked secret, turn on the secret checkbox.
	CtlSetValue(GetObjectPtr(MemoPadDetailsPrivateCheckbox), secret);

	// P11. Set the label of the category trigger.
	CategoryGetName (MemoPadDB, RecordCategory, CategoryName);
	ctl = GetObjectPtr (MemoPadDetailsCategoryPopTrigger);
	CategorySetTriggerLabel (ctl, CategoryName);
}


/***********************************************************************
 *
 * FUNCTION:    DetailsFormHandleEvent
 *
 * DESCRIPTION: P10. Handles processing of events for the details form.
 *
 * PARAMETERS:  event - the most recent event.
 *
 * RETURNED:    True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean DetailsFormHandleEvent(EventPtr event)
{
	FormPtr		frm;
	Boolean		handled = false;
	Int 			updateCode;				// P11. updates for the edit form

	if (event->eType == ctlSelectEvent)
		{
   	// A control button was pressed and released.	
		switch (event->data.ctlSelect.controlID)
			{
			case MemoPadDetailsOKButton:
	   		// The ok button was pressed so apply the changes.
				updateCode = DetailsApply();
				
				// Remove the details form and display the edit form.
				FrmReturnToForm(MemoPadEditForm);
				if (updateCode)
					FrmUpdateForm (MemoPadEditForm, updateCode);
				handled = true;
				break;

			case MemoPadDetailsCancelButton:
	   		// The cancel button was pressed, just go back to the edit form.
				FrmReturnToForm(MemoPadEditForm);
				handled = true;
				break;
				
			case MemoPadDetailsDeleteButton:
	   		// The delete button was pressed.  Delete the current record.
				if (EditViewDeleteCurrentRecord())
					{
					// Delete was completed.  Close the details and the edit forms.
					FrmCloseAllForms();
					
					// Go to the main form.
					FrmGotoForm(MemoPadMainForm);
					}
				handled = true;
				break;
			
			case MemoPadDetailsCategoryPopTrigger:
				// P11. The category trigger was pressed.  Have the
				// system ui present a list of categories and return
				// what the user selects.  The categories listed are those
				// found in the app info block.
				if (CategorySelect (MemoPadDB, FrmGetActiveForm (),
					MemoPadDetailsCategoryPopTrigger, MemoPadDetailsCategoriesListList,
					false, &NewCategory, CategoryName, 1, 0))
					{
					NewCategoryEdited = true;
					}
				handled = true;
				break;

			}
		}

	else if (event->eType == frmOpenEvent)
		{
  		// The form was told to open.  It has already been loaded and activated.
		frm = FrmGetActiveForm();
		
  		// Initialize the details form.
		DetailsInit();
		
		// Draw the details form.
		FrmDrawForm(frm);
		handled = true;
		}

	return(handled);
}


/***********************************************************************
 *
 * FUNCTION:    EditViewSetTitle
 *
 * DESCRIPTION: P16. This routine formats and sets the title of the edit view.
 *              If the edit view is visible, the new title is drawn.
 *
 *					 The title parts are referenced as follows:
 *					 [text 1][arg 1][text 2][arg 2][text 3]
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
 
static void EditViewSetTitle (void)
{
	CharPtr titleTemplateP;
	CharPtr textStart;		// start of the text part we copy
	CharPtr textEnd;			// end of the text part we copy
	CharPtr endOfNewTitle;
	FormPtr frm;
	char posStr [digitsForRecordPosition + 1];
	char totalStr [digitsForRecordPosition + 1];
	UInt pos;
	UInt length;
	

	// Format as strings, the memo's postion within its category, and 
	// the total number of memos in the category.
	pos = DmPositionInCategory (MemoPadDB, CurrentRecord, RecordCategory);
	StrIToA (posStr, pos+1);

	// P16. Count the records if unknown.  Remember this routine takes account
	// of private records.
	if (MemosInCategory == memosInCategoryUnknown)
		MemosInCategory = DmNumRecordsInCategory (MemoPadDB, RecordCategory);
	StrIToA (totalStr, MemosInCategory);


	// Get the title template string.  It contains '#' chars which we replace
	// with the position of CurrentRecord within CurrentCategory and with
	// the total count of records in CurrentCategory ().
	titleTemplateP = MemHandleLock (DmGetResource (strRsc, EditViewTitleTemplateStringString));


	// Free any old title.  Also done when the form is closed in EditViewHandleEvent.
	if (EditViewTitlePtr)
		MemPtrFree(EditViewTitlePtr);

	// Now get space for the new title.  Subtract 2 (the number of '#' chars).
	length = StrLen (posStr) + StrLen (totalStr) + StrLen (titleTemplateP) - 2;
	EditViewTitlePtr = MemPtrNew (length);
	ErrFatalDisplayIf (!EditViewTitlePtr, "Out of memory");
	endOfNewTitle = EditViewTitlePtr;
	

	// Copy text 1
	textStart = titleTemplateP;
	textEnd = StrChr (textStart, titleReplacementChar);
	length = textEnd - textStart;
	MemMove (endOfNewTitle, textStart, length);
	endOfNewTitle[length] = 0;
	endOfNewTitle += length;
	
	// Copy arg 1
	StrCopy(endOfNewTitle, posStr);
	endOfNewTitle += StrLen(endOfNewTitle);

	
	// Copy text 2
	textStart = textEnd + sizeof((char) titleReplacementChar);
	textEnd = StrChr (textStart, titleReplacementChar);
	length = textEnd - textStart;
	MemMove (endOfNewTitle, textStart, length);
	endOfNewTitle[length] = 0;
	endOfNewTitle += length;
	
	// Copy arg 2
	StrCopy(endOfNewTitle, totalStr);
	endOfNewTitle += StrLen(endOfNewTitle);

	
	// Copy text 3
	textStart = textEnd + sizeof((char) titleReplacementChar);
	StrCat(endOfNewTitle, textStart);

	
	// Now set the title to use the new title string.
	frm = FrmGetFormPtr (MemoPadEditForm);
	FrmSetTitle (frm, EditViewTitlePtr);
	MemPtrUnlock(titleTemplateP);
}


/***********************************************************************
 *
 * FUNCTION:    EditViewUpdateScrollBar
 *
 * DESCRIPTION: P13. Visually updates the scroll bar.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void EditViewUpdateScrollBar ()
{
	Word scrollPos;
	Word textHeight;
	Word fieldHeight;
	Short maxValue;
	FieldPtr fld;
	ScrollBarPtr bar;

	fld = GetObjectPtr (MemoPadEditEditFieldField);
	bar = GetObjectPtr (MemoPadEditScrollbarScrollBar);
	
	// get the values necessary to update the scroll bar.
	FldGetScrollValues (fld, &scrollPos, &textHeight,  &fieldHeight);

	if (textHeight > fieldHeight)
		maxValue = textHeight - fieldHeight;
	else if (scrollPos)
		maxValue = scrollPos;
	else
		maxValue = 0;

	SclSetScrollBar (bar, scrollPos, 0, maxValue, fieldHeight-1);
}


/***********************************************************************
 *
 * FUNCTION:    EditViewUpdateDisplay
 *
 * DESCRIPTION: P11. This routine updates the display of the edit view
 *
 * PARAMETERS:  updateCode - a code that indicated what changes have been
 *                           made to the view.
 *                		
 * RETURNED:    nothing
 *
 ***********************************************************************/
static Boolean EditViewUpdateDisplay (Int updateCode)
{
	FormPtr frm;

	frm = FrmGetActiveForm ();

	if (updateCode == updateCategoryChanged)
		{
		EditViewSetTitle ();				// P16.  Update the title to display the
												// position of the record in the new category
		
		
		// Set the label of the category trigger.
		CategoryGetName (MemoPadDB, RecordCategory, CategoryName);
		CategorySetTriggerLabel (GetObjectPtr (MemoPadEditCategoryPopTrigger),
			CategoryName);
		
		// If the record won't be visible in the main form change the
		// category to keep it visible.  If the user doesn't see the 
		// record they might fear it lost.
		if (CurrentCategory != dmAllCategories &&
			CurrentCategory != RecordCategory)
			CurrentCategory = RecordCategory;
			
		return (true);
		}
	
	return (false);
}


/***********************************************************************
 *
 * FUNCTION:    EditViewSelectCategory
 *
 * DESCRIPTION: P11. This routine changes the category of the memo 
 * 				 in the edit form.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 *              The following global variables are modified:
 *							CurrentCategory
 *							CategoryName
 *
 ***********************************************************************/
static void EditViewSelectCategory (void)
{
	UInt attr;
	FormPtr frm;
	UInt category;
	Boolean categorySelected;
	Boolean categoryEdited;


	// Get the current category.	
	category = RecordCategory;
	
	// Process the category popup list.
	frm = FrmGetActiveForm();
	
	// Have the system ui present the user with a list of categories
	// and return what the user selects.
	categoryEdited = CategorySelect (MemoPadDB, frm, MemoPadEditCategoryPopTrigger,
					  MemoPadEditCategoriesListList, false, &category, CategoryName, 1, 0);


	categorySelected = (category != RecordCategory);

	// If a different category was selected, set the category field
	// in the new record.
	if (categorySelected)
		{
		// Change the category of the record.
		DmRecordInfo (MemoPadDB, CurrentRecord, &attr, NULL, NULL);	
		attr &= ~dmRecAttrCategoryMask;
		attr |= (category | dmRecAttrDirty);
		DmSetRecordInfo (MemoPadDB, CurrentRecord, &attr, NULL);
		}


	// If the current category was changed or the name of the category 
	// was edited,  draw the title.
	if (categoryEdited || categorySelected)
		{
		ChangeCategory (category);
		EditViewSetTitle ();				// P16.  Update the title to display the
												// position of the record in the new category
		}
}


/***********************************************************************
 *
 * FUNCTION:    EditViewChangeFont
 *
 * DESCRIPTION: P16. Change the font used by the field in the edit view.
 *					 Redisplay the field and update the scroll arrows.
 *
 * PARAMETERS:  controlID - id to button pressed.
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void EditViewChangeFont (UInt controlID)
{
	FieldPtr fld;
	
	
	fld = GetObjectPtr (MemoPadEditEditFieldField);

	if (controlID == MemoPadEditLargeFontPushButton)
		EditViewFont = largeFont;
	else
		EditViewFont = stdFont;
		
	// FldSetFont will redraw the field if it is visible.
	FldSetFont (fld, EditViewFont);

	// Update the ScrollBar because changing the font changes where the
	// words wrap and therefore the layout of the field.  Also, the field
	// will be longer so the field may now be scrollable when it might
	// not have been before.
	EditViewUpdateScrollBar ();
}


/***********************************************************************
 *
 * FUNCTION:    EditViewDeleteCurrentRecord
 *
 * DESCRIPTION: P10. This routine deletes the current memo record following
 *              confirmation from the user.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    True if the record is deleted, false otherwise.
 *
 ***********************************************************************/
static Boolean EditViewDeleteCurrentRecord(void)
{
	FormPtr  frm;
	FieldPtr fld;

	// Display alert to get confirmation for the delete request.
	if (FrmAlert(DeleteAlertAlert) == DeleteAlertOK)
		{
		// The delete has been confirmed.
		// Get a pointer to the field.
		frm = FrmGetFormPtr(MemoPadEditForm);
		fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MemoPadEditEditFieldField));
		
		// Delete the text in the field, if there is any.
		if (FldGetTextPtr(fld))
			FldDelete(fld, 0, FldGetTextLength(fld));
		
		// Inform the caller that the delete was completed.
		return(true);
		}
	
	else 
		// Inform the caller that the delete was cancelled.
		return(false);
}


/***********************************************************************
 *
 * FUNCTION:		EditViewSaveData
 *
 * DESCRIPTION:	P5. Saves the data in the field as the first record in 
 *                the application's database.
 *
 * PARAMETERS:		fld - the field that contains the text.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void EditViewSaveData(FieldPtr fld)
{
	CharPtr	text;
	UInt attr;
	UInt mode;

	
	// Get a pointer to the field's text.
	text = FldGetTextPtr(fld);

	// P6. Clear the handle value in the field, otherwise the handle
	// will be freed when the form is closed.
	// This will cause problems since the handle is to the data
	// IN the database record, not a separate copy.
	FldSetTextHandle(fld, 0);

	// Is there data in the field?
	if (text != NULL && *text != 0)
		{
		// Check to see if the record disappears because it's private
		// and private records are ignored.
		DmRecordInfo (MemoPadDB, CurrentRecord, &attr, NULL, NULL);
		
		// There are three ways to find out if private records are 
		// hidden:
		// 1) query the database to see how it was opened
		// 2) query the system preferences
		// 3) keep the value from StartApplication in a global variable.
		DmOpenDatabaseInfo(MemoPadDB, NULL, NULL, &mode, NULL, NULL);
		
		if ((mode & dmModeShowSecret) && 
			(attr & dmRecAttrSecret) &&
			MemosInCategory != memosInCategoryUnknown)
			{
			MemosInCategory--;			
			}


		// Release the record to the database manager.  The true value indicates that
		// the record contains "dirty" data.  Release Record will set the record's 
		// dirty flag and update the database modification count.
		DmReleaseRecord(MemoPadDB, CurrentRecord, true);
		}
	else 
		{
		// P7. The record is empty, remove it from the database.
		DmDeleteRecord(MemoPadDB, CurrentRecord); // P17. Delete instead of remove
																// so the record gets deleted
																// on the desktop during sychronization.
		CurrentRecord = noRecordSelected;
		if (MemosInCategory != memosInCategoryUnknown)
			MemosInCategory--;
		}
}


/***********************************************************************
 *
 * FUNCTION:		EditViewRetrieveData
 *
 * DESCRIPTION:	P5. Retrieves the data from the first record of a database 
 *						and places it in a field.
 *
 * PARAMETERS:		fld - the field to contain the text.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void EditViewRetrieveData(FieldPtr fld)
{
	VoidHand	recHandle;
	UInt attr;
	
	
	// Check to make sure there is a record to retrieve.
	if (CurrentRecord != noRecordSelected && DmNumRecords(MemoPadDB) > 0)
		{
		// Get a handle for the first record.
		recHandle = DmGetRecord(MemoPadDB, CurrentRecord);
		
		// P6. Set the field's text to the data in the record.
		FldSetTextHandle(fld, recHandle);

		// P16. Set the font
		FldSetFont (fld, EditViewFont);

		// P11. Set the global variable that keeps track of the current category
		// to the category of the current record.
		DmRecordInfo (MemoPadDB, CurrentRecord, &attr, NULL, NULL);
		RecordCategory = attr & dmRecAttrCategoryMask;
		
		// P11. Set the label that contains the note's category.
		CategoryGetName (MemoPadDB, RecordCategory, CategoryName);
		CategorySetTriggerLabel (GetObjectPtr (MemoPadEditCategoryPopTrigger), CategoryName);
	
		EditViewSetTitle ();										// P16.

		EditViewUpdateScrollBar ();	// P13
		}
}



/***********************************************************************
 *
 * FUNCTION:    EditViewPageScroll
 *
 * DESCRIPTION: P13. Scrolls the edit view a page at a time.
 *
 * PARAMETERS:   direction - up or down
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void EditViewPageScroll (DirectionType direction)
{
	Short value;
	Short min;
	Short max;
	Short pageSize;
	Word linesToScroll;
	FieldPtr fld;
	ScrollBarPtr bar;

	fld = GetObjectPtr (MemoPadEditEditFieldField);
	
	if (FldScrollable (fld, direction))
		{
		linesToScroll = FldGetVisibleLines (fld) - 1;
		FldScrollField (fld, linesToScroll, direction);

		// Update the scroll bar.
		bar = GetObjectPtr (MemoPadEditScrollbarScrollBar);
		SclGetScrollBar (bar, &value, &min, &max, &pageSize);

		if (direction == up)
			value -= linesToScroll;
		else
			value += linesToScroll;
		
		SclSetScrollBar (bar, value, min, max, pageSize);
		}
}



/***********************************************************************
 *
 * FUNCTION:    EditViewScroll
 *
 * DESCRIPTION: P13.  Scrolls the edit view a line at a time.
 *
 * PARAMETERS:  linesToScroll - number of lines to scroll, 
 *                              negative numbers are used to scroll up
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void EditViewScroll (Short linesToScroll)
{
	Word				blankLines;
	Short				min;
	Short				max;
	Short				value;
	Short				pageSize;
	FieldPtr			fld;
	ScrollBarPtr	bar;
	
	fld = GetObjectPtr (MemoPadEditEditFieldField);

	if (linesToScroll < 0)
		{
		blankLines = FldGetNumberOfBlankLines (fld);
		FldScrollField (fld, -linesToScroll, up);
		
		// If there were blank lines visible at the end of the field
		// then we need to update the scroll bar.
		if (blankLines)
			{
			// Update the scroll bar.
			bar = GetObjectPtr (MemoPadEditScrollbarScrollBar);
			SclGetScrollBar (bar, &value, &min, &max, &pageSize);
			if (blankLines > -linesToScroll)
				max += linesToScroll;
			else
				max -= blankLines;
			SclSetScrollBar (bar, value, min, max, pageSize);
			}
		}

	else if (linesToScroll > 0)
		FldScrollField (fld, linesToScroll, down);
}


/***********************************************************************
 *
 * FUNCTION:    EditViewDoMenuCommand
 *
 * DESCRIPTION: P4. This routine performs the menu command specified.
 *
 * PARAMETERS:  command - menu item id
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void EditViewDoMenuCommand(Int command)
{
	FieldPtr	fld;
	FormPtr	frm;

	// Do the appropriate action for the menu command selected.
	switch (command)
		{
		case EditPageNewPage:		// P9. Create a new memo page.
			// Save the current data in the field.
			fld = GetFocusObjectPtr();
			if (fld)
			{
   			EditViewSaveData(fld);

				// Clear the existing text in the field.
  				FldDelete(fld, 0, FldGetTextLength(fld));
   			
				// Create a new record in the database.
				if (CreateRecord())
					{
	   			// Get the data for the field from the application's database.
   				EditViewRetrieveData(fld);
   			
   				// Draw the new field data
   				FldDrawField(fld);
   				}
   			}
			break;
		
		case EditPageDeletePage:	// P9. Delete the current memo page.
			// P10. Delete the current database record.
			if (EditViewDeleteCurrentRecord())
	   		// Go back to the main form.
				FrmGotoForm(MemoPadMainForm);
			break;
			
		case EditOptionsGetInfo:		// Display the info form.
			// Load the info form, then display it.
			frm = FrmInitForm(MemoPadInfoForm);
			FrmDoDialog(frm);
			
			// Delete the info form.
	 		FrmDeleteForm(frm);
			break;
		
		case EditOptionsGototopofpage:		// Go to the top of the field.	
			fld = GetFocusObjectPtr();
			if (fld)
				{
				FldSetInsPtPosition(fld, 0);
				EditViewUpdateScrollBar ();	// P13
				}
			break;
		
		case EditOptionsGotobottomofpage:	// Go to the bottom of the field.
			fld = GetFocusObjectPtr();
			if (fld)
				{
				FldSetInsPtPosition(fld, FldGetTextLength(fld));
				EditViewUpdateScrollBar ();	// P13
				}

			break;
		
		case EditEditCut:			// Do a cut to the clipboard if text is highlighted.
			fld = GetFocusObjectPtr();
			if (fld)
				{
				FldCut(fld);
				EditViewUpdateScrollBar ();	// P13
				}
			break;
		
		case EditEditCopy:			// Copy the highlighted text to the clipboard.
			fld = GetFocusObjectPtr();
			if (fld)
				{
				FldCopy(fld);	
				}
			break;
		
		case EditEditPaste:			// Do a paste from the clipboard.
			fld = GetFocusObjectPtr();
			if (fld)
				{
				FldPaste(fld);		
				EditViewUpdateScrollBar ();	// P13
				}
			break;
			
		case EditEditUndo:			// Undo the last text change.
			fld = GetFocusObjectPtr();
			if (fld)
				{
				FldUndo(fld);
				EditViewUpdateScrollBar ();	// P13
				}
			break;
		
		case EditEditSelectAll:	// Select all the existing text.
			fld = GetFocusObjectPtr();
			if (fld)
				FldSetSelection(fld, 0, FldGetTextLength(fld));
			break;
			
		case EditEditKeyboard:		// Display the on screen keyboard.
			SysKeyboardDialog(kbdAlpha);
			break;

		case EditEditGraffiti:		// Display the graffiti reference screen
			SysGraffitiReferenceDialog (referenceDefault);
			break;		
		}	
}


/***********************************************************************
 *
 * FUNCTION:		EditViewHandleEvent
 *
 * DESCRIPTION:	P3. Handles processing of events for the edit form.
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean EditViewHandleEvent(EventPtr event)
{
	FormPtr	frm;
	Int		fldIndex;
	FieldPtr	fldP;					// P14.
	Boolean	handled = false;
	Int		controlID;


	switch (event->eType)
	{
   	case keyDownEvent:  // P13. User used Graffiti or a physical button
   		if (event->data.keyDown.chr == pageUpChr)
				{
				EditViewPageScroll (up);
				handled = true;
				}
	
			else if (event->data.keyDown.chr == pageDownChr)
				{
				EditViewPageScroll (down);
				handled = true;
				}
	
			else
				{
				// An ordinary ASCII character was entered.  Have the form
				// give the field the character.  Then check to see if the 
				// scrolling changed.
				frm = FrmGetActiveForm ();
				FrmHandleEvent (frm, event);
				EditViewUpdateScrollBar ();
				handled = true;
				}
			break;
   	
			
		case sclRepeatEvent:  // P13. the scroll bar was pressed
			EditViewScroll (event->data.sclRepeat.newValue - 
				event->data.sclRepeat.value);

	   	// Repeating controls don't repeat if handled is set true.
			break;
			
			
   	case ctlSelectEvent:  // A control button was pressed and released.
			
	   	// If the done button is pressed, go back to the main form.
	   	if (event->data.ctlEnter.controlID == MemoPadEditDoneButton)
	   		{
				// Remove the edit form and display the main form.
	   		FrmGotoForm(MemoPadMainForm);
				handled = true;
				}

	   	// P10. If the details button is pressed, pop up the details form.
			else if (event->data.ctlEnter.controlID == MemoPadEditDetailsButton)
				{
				FrmPopupForm(MemoPadDetailsForm);
				handled = true;
				}

	   	// P11. If the category trigger is pressed, pop up the category list.
			else if (event->data.ctlEnter.controlID == MemoPadEditCategoryPopTrigger)
				{
				EditViewSelectCategory ();
				handled = true;
				}

	   	// P16. If a font button is pressed, change the field's font.
			else if (event->data.ctlEnter.controlID == MemoPadEditSmallFontPushButton ||
				event->data.ctlEnter.controlID == MemoPadEditLargeFontPushButton)
				{
				EditViewChangeFont (event->data.ctlSelect.controlID);
				handled = true;
				}
			break;
  	

  		case frmUpdateEvent:	// P11. The form must update it's presentation
			handled =  EditViewUpdateDisplay (event->data.frmUpdate.updateCode);
			break;


  		case frmOpenEvent:	// The form was told to open.
  			// It has already been loaded and activated.
  			
  			// P5. Get a pointer to the form and the index of the text field.
			frm = FrmGetActiveForm();
			fldIndex = FrmGetObjectIndex(frm, MemoPadEditEditFieldField);

			//	P16. Highlight the appropriate font push button
			if (EditViewFont == stdFont)
				controlID = MemoPadEditSmallFontPushButton;
			else
				controlID = MemoPadEditLargeFontPushButton;
			FrmSetControlGroupSelection (frm, editFontGroup, controlID);

   		// P5. Get the data for the field from the application's database.
   		EditViewRetrieveData(FrmGetObjectPtr(frm, fldIndex));
			
  			// Draw the form with the text field data in place.
			FrmDrawForm(frm);
			
			// P4. Set the focus, and the insertion point, to the text edit field.
			FrmSetFocus(frm, fldIndex);
			handled = true;
			break;
			
  		case frmGotoEvent:	// P14. The form should goto a record.
  			// The form has already been loaded and activated.
  			
  			// Get a pointer to the form and the index of the text field.
			frm = FrmGetActiveForm();
			fldIndex = FrmGetObjectIndex(frm, MemoPadEditEditFieldField);
			fldP = FrmGetObjectPtr(frm, fldIndex);

			//	P16. Highlight the appropriate font push button
			if (EditViewFont == stdFont)
				controlID = MemoPadEditSmallFontPushButton;
			else
				controlID = MemoPadEditLargeFontPushButton;
			FrmSetControlGroupSelection (frm, editFontGroup, controlID);

   		// Get the data for the field from the application's database.
			CurrentRecord = event->data.frmGoto.recordNum;
   		EditViewRetrieveData(fldP);
			
			// Scroll to the found text and select it.
			FldSetScrollPosition(fldP, event->data.frmGoto.matchPos);
			FldSetSelection(fldP, event->data.frmGoto.matchPos, 
				event->data.frmGoto.matchPos + event->data.frmGoto.matchLen);
			EditViewUpdateScrollBar ();

  			// Draw the form with the text field data in place.
			FrmDrawForm(frm);
			
			// Set the focus, and the insertion point, to the text edit field.
			FrmSetFocus(frm, fldIndex);
			handled = true;
			break;
			
		case menuEvent:		// P4. A menu item was selected.
			// First clear the menu status from the display.
			MenuEraseStatus(0);
			
			// Process menu commands for the edit form.
			EditViewDoMenuCommand(event->data.menu.itemID);
			handled = true;
			break;

  		case frmCloseEvent:
  			// P5. Get a pointer to the form and the index of the text field.
   		frm = FrmGetActiveForm();
   		fldIndex = FrmGetObjectIndex(frm, MemoPadEditEditFieldField);
   		
   		// P5. Save the data from the field to the application's database.
   		EditViewSaveData(FrmGetObjectPtr(frm, fldIndex));
   		
			// P16. Free any title.
			if (EditViewTitlePtr)
				{
				MemPtrFree(EditViewTitlePtr);
				EditViewTitlePtr = NULL;
				}
			break;
	}
	return(handled);
}


/***********************************************************************
 *
 * FUNCTION:    MainViewUpdateScrollers
 *
 * DESCRIPTION: P13. This routine draws or erases the list view scroll arrow
 *              buttons.
 *
 * PARAMETERS:  frm          -  pointer to the to do list form
 *              bottomRecord -  record index of the last visible record
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void MainViewUpdateScrollers (FormPtr frm, Int bottomRecord)
{
	Int upIndex;
	Int downIndex;
	Word recordNum;
	Boolean scrollableUp;
	Boolean scrollableDown;
		
	// If the first record displayed is not the fist record in the category,
	// enable the up scroller.
	recordNum = TopVisibleRecord;
	scrollableUp = !DmSeekRecordInCategory (MemoPadDB, &recordNum, 1, 
		dmSeekBackward, CurrentCategory);


	// If the last record displayed is not the last record in the category,
	// enable the down scroller.
	recordNum = bottomRecord;
	scrollableDown = !DmSeekRecordInCategory (MemoPadDB, &recordNum, 1, 
		dmSeekForward, CurrentCategory); 


	// Update the scroll button.
	upIndex = FrmGetObjectIndex (frm, MemoPadMainScrollUpRepeating);
	downIndex = FrmGetObjectIndex (frm, MemoPadMainScrollDownRepeating);
	FrmUpdateScrollers (frm, upIndex, downIndex, scrollableUp, scrollableDown);
}


/***********************************************************************
 *
 * FUNCTION:		MainViewDrawRecordInBounds
 *
 * DESCRIPTION:	P14. Draw an item within a passed bounds
 *
 * PARAMETERS:		memoPadRecP - the record to draw
 *						bounds - bounds in which to draw the record
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void MainViewDrawRecordInBounds(MemoPadDBRecordTypePtr memoPadRecP, 
	RectanglePtr bounds)
{
	Int			textLen, width;
	Boolean		fits;
	FontID 		currFont;
	

	// Set the standard font.  Save the current font.
	// It is a Pilot convention to not destroy the current font
	// but to save and restore it.
	currFont = FntSetFont (stdFont);

	// Determine the length of text that will fit within the bounds.
	width = bounds->extent.x - 2;
	textLen = StrLen(memoPadRecP);
	FntCharsInWidth(memoPadRecP, &width, &textLen, &fits);
	
	// Now draw the text from the record.
	WinDrawChars(memoPadRecP, textLen, bounds->topLeft.x, bounds->topLeft.y);
	
	// Restore the font.
	FntSetFont (currFont);
}


/***********************************************************************
 *
 * FUNCTION:		MainViewDrawRecord
 *
 * DESCRIPTION:	P12. Draw an item in the main form's table.  This 
 *						routine is called from the table object and must match
 *						the parameters that the table object passes.  The routine
 *						MainViewLoadRecords sets the table object to call this
 *						routine.  The table object calls it once for every table 
 *						cell needing drawing.
 *
 * PARAMETERS:		tableP - table in which to draw the record
 *						row - row of the record to change
 *						column - column of the record to change
 *						bounds - bounds in which to draw the record
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void MainViewDrawRecord(VoidPtr tableP, Word row, Word column, 
	RectanglePtr bounds)
{
	Handle		recHandle;
	CharPtr		recText;
	UInt			recordNum = 0;
	

	// Get the record number that corresponds to the table item to draw.
	// The record number is stored in the "intValue" field of the item.
	recordNum = TblGetItemInt (tableP, row, column);

	// Retrieve the record from the database and lock it down.
	recHandle = DmGetRecord(MemoPadDB, recordNum);
	recText = MemHandleLock(recHandle);

	// P14. Draw the record
	MainViewDrawRecordInBounds(recText, bounds);
	
	// Unlock the handle to the record.
	MemHandleUnlock(recHandle);
	
	// Release the record, not dirty.
	DmReleaseRecord(MemoPadDB, recordNum, false);
}


/***********************************************************************
 *
 * FUNCTION:    MainViewLoadTable
 *
 * DESCRIPTION: P12. Loads the database records into the main table.
 *
 * PARAMETERS:  frm - form containing table to load
 *					 recordNum - index of the first record to display.
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void MainViewLoadTable (FormPtr frm, UInt recordNum)
{
	Word			row;
	Word			lastRecordNum;
	Word			numRows;
	TablePtr 	tableP;
	VoidHand		recordH;

	
	tableP = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, MemoPadMainMemoTableTable));

	// For each row in the table, store the record number in the 
	// table item that will display the record.  
	numRows = TblGetNumberOfRows (tableP);
	for (row = 0; row < numRows; row++, recordNum++)
		{		
		// Get the next record in the current category.
		recordH = DmQueryNextInCategory (MemoPadDB, &recordNum, 
			CurrentCategory);
		
		// If the record was found, store the record number in the table item,
		// otherwise set the table row unusable.
		if (recordH)
			{
			// Set customTableItem to indicate that we want to be called
			// to draw the record.  The callback set in MainViewLoadRecords
			// will be called to draw the record.
			TblSetItemStyle (tableP, row, 0, customTableItem);
			TblSetItemInt (tableP, row, 0, recordNum);
			TblSetRowUsable (tableP, row, true);
			lastRecordNum = recordNum;
			}
		else
			{
			// Sometimes there are more table rows than records.
			// If this happens mark those unused table rows as
			// not usable.
			TblSetRowUsable (tableP, row, false);
			}

		// Mark the row invalid so that it will draw when we call the 
		// draw routine.
		TblMarkRowInvalid (tableP, row);

		}

	// P13. Update the scroll arrows.
	MainViewUpdateScrollers (frm, lastRecordNum);
}


/***********************************************************************
 *
 * FUNCTION:		MainViewLoadRecords
 *
 * DESCRIPTION:	P12. Loads the table object with records.
 *
 * PARAMETERS:		frm - form containing table to load
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void MainViewLoadRecords(FormPtr frm)
{
	TablePtr 	tableP;
	Word			recordNum;
	Word			rowsInTable;

	
	// Get the table and it's number of rows
	tableP = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, MemoPadMainMemoTableTable));
	rowsInTable = TblGetNumberOfRows (tableP);

	// Before we load the table with records we should do any positioning
	// of the table now.
	
	// If there is a current record is it visible?
	if (CurrentRecord != noRecordSelected)
		{
		// Is the current record before the first visible record?
		if (TopVisibleRecord > CurrentRecord)
			{
			// Make CurrentRecord the top record displayed.
			TopVisibleRecord = CurrentRecord;
			}
		
		// Is the current record after the last visible record?
		else
			{
			// Find the record displayed at the bottom of the table.
			// If the record is before the current record then the 
			// current record isn't displayed.  In that case make
			// the CurrentRecord the first record displayed in the table.
			recordNum = TopVisibleRecord;
			DmSeekRecordInCategory (MemoPadDB, &recordNum, rowsInTable-1,
				dmSeekForward, CurrentCategory);
			if (recordNum < CurrentRecord)
				TopVisibleRecord = CurrentRecord;
			}
		}

	
	// Try to show a full display of records.  Starting at the last
	// record and working backwards, find the record displayed at the 
	// top of the table.  If the record is before the TopVisibleRecord
	// then TopVisibleRecord is set too far down the list of records.
	// Set TopVisibleRecord to the record one screen full from the end.
	recordNum = dmMaxRecordIndex;
	DmSeekRecordInCategory (MemoPadDB, &recordNum, (rowsInTable-1),
		dmSeekBackward, CurrentCategory);
	TopVisibleRecord = min (TopVisibleRecord, recordNum);


	// Now that the table is in position load the records.
	MainViewLoadTable (frm, TopVisibleRecord);

	// Set the callback routine that will draw the records.
	TblSetCustomDrawProcedure (tableP, 0, MainViewDrawRecord);

	// Set the column usable so that it draws and accepts user input.
	TblSetColumnUsable (tableP, 0, true);	
}


/***********************************************************************
 *
 * FUNCTION:    MainViewSelectCategory
 *
 * DESCRIPTION: P11. This routine handles selection, creation and deletion of
 *              categories for the Main View.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    The index of the new category.
 *
 *              The following global variables are modified:
 *							CurrentCategory
 *							CategoryName
 *
 ***********************************************************************/
static Int MainViewSelectCategory (void)
{
	FormPtr frm;
	Word category;
	Boolean categoryEdited;
	TablePtr tableP;

	
	// Process the category popup list.  
	category = CurrentCategory;

	frm = FrmGetActiveForm();
	categoryEdited = CategorySelect (MemoPadDB, frm, MemoPadMainCategoryPopTrigger,
					  MemoPadMainCategoriesListList, true, &category, CategoryName, 1, 0);
	
	if (categoryEdited || (category != CurrentCategory))
		{
		CurrentCategory = category;

		// P16. Count the records later when needed to display.
		MemosInCategory = memosInCategoryUnknown;
	
		// Display the new category.
		MainViewLoadRecords (frm);
		tableP = GetObjectPtr (MemoPadMainMemoTableTable);
		TblEraseTable (tableP);
		TblDrawTable (tableP);
		}

	return (category);
}


/***********************************************************************
 *
 * FUNCTION:    MainViewScroll
 *
 * DESCRIPTION: P13. This routine scrolls the list of memo titles
 *              in the direction specified.
 *
 *					 Scrolling up stops at the first record visible.  
 *					 Because of categories and private records the first
 *					 record visible isn't neccessarily record 0.
 *
 *					 Scrolling down stops when less than a full table of
 *					 records can be displayed.  To enfore this when the 
 *					 table is scrolled down, we check if at the new 
 *					 position there are enough records visible to fill
 *					 the table.  If not we find the last records visible
 *					 by working backwards from the end.
 *
 * PARAMETERS:  direction - up or dowm
 *					 oneLine - true scrolls one line, false scrolls one page
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void MainViewScroll (DirectionType direction, Boolean oneLine)
{
	TablePtr tableP;
	Int 		rowsInTable;
	UInt		newTopVisibleRecord;
	FormPtr	frmP;

	tableP = GetObjectPtr (MemoPadMainMemoTableTable);
	rowsInTable = TblGetNumberOfRows (tableP);
	newTopVisibleRecord = TopVisibleRecord;
	CurrentRecord = noRecordSelected;

	// Scroll the table down.
	if (direction == down)
		{
		// Scroll down a single line.
		if (oneLine)
			{
			DmSeekRecordInCategory (MemoPadDB, &newTopVisibleRecord, 1, 
				dmSeekForward, CurrentCategory);
			}

		// Scroll down a page (less one row).
		else
			{
			// Try going forward one page
			if (DmSeekRecordInCategory (MemoPadDB, &newTopVisibleRecord, 
					rowsInTable - 1, dmSeekForward, CurrentCategory))
				{
				// Try going backwards one page from the last record
				newTopVisibleRecord = dmMaxRecordIndex;
				DmSeekRecordInCategory (MemoPadDB, &newTopVisibleRecord, 
					rowsInTable - 1, dmSeekBackward, CurrentCategory);
				}
			}
		}



	// Scroll the table up.
	else
		{
		// Scroll up a single line
		if (oneLine)
			{
			DmSeekRecordInCategory (MemoPadDB, &newTopVisibleRecord, 1, 
				dmSeekBackward, CurrentCategory);
			}

		// Scroll up a page (less one row).
		else
			{
			if (DmSeekRecordInCategory (MemoPadDB, &newTopVisibleRecord, 
				rowsInTable - 1, dmSeekBackward, CurrentCategory))
				{
				// Not enough records to fill one page.  Start with the first record
				newTopVisibleRecord = 0;
				DmSeekRecordInCategory (MemoPadDB, &newTopVisibleRecord, 0, 
					dmSeekForward, CurrentCategory);
				}
			}
		}


	// Avoid redraw if no change
	if (TopVisibleRecord != newTopVisibleRecord)
		{
		// The table should be at a different position.  Load the
		// table with new records and redraw it.
		TopVisibleRecord = newTopVisibleRecord;
		frmP = FrmGetActiveForm ();
		MainViewLoadRecords (frmP);
		TblRedrawTable(tableP);
		}
}


/***********************************************************************
 *
 * FUNCTION:		MainViewInit
 *
 * DESCRIPTION:	P7. Initializes the main form and the list object.
 *
 * PARAMETERS:		frm - pointer to the main form.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void MainViewInit(void)
{
	FormPtr			frm;


	// Get a pointer to the main form.
	frm = FrmGetActiveForm();
	
	MainViewLoadRecords(frm);

	// P11. Set the label of the category trigger.
	CategoryGetName (MemoPadDB, CurrentCategory, CategoryName);
	CategorySetTriggerLabel (GetObjectPtr (MemoPadMainCategoryPopTrigger), CategoryName);

	// Draw the form.
	FrmDrawForm(frm);
}


/***********************************************************************
 *
 * FUNCTION:		MainViewHandleEvent
 *
 * DESCRIPTION:	Handles processing of events for the main form.
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean MainViewHandleEvent(EventPtr event)
{
	Boolean		handled = false;
	FormPtr		frm;


	switch (event->eType)
		{
   	case ctlSelectEvent:  // A control button was pressed and released.
	   	if (event->data.ctlEnter.controlID == MemoPadMainNewButton)
	   		{
				// P5. Flag to the edit form that a new record is needed.
				// CurrentRecord = noRecordSelected;
				
				// P6. Create a new database record.
				if (CreateRecord())
					{
					// P3. The new button is pressed, go to the edit form.
					FrmGotoForm(MemoPadEditForm);
					}
				handled = true;
				}
			
	   	// P11. If the category trigger is pressed, pop up the category list.
	   	else if (event->data.ctlEnter.controlID == MemoPadMainCategoryPopTrigger)
	   		{
				MainViewSelectCategory();
				handled = true;
				}
			break;
  	

   	case ctlRepeatEvent:		// P13.  an onscreen scroll button is pressed
	   	if (event->data.ctlRepeat.controlID == MemoPadMainScrollUpRepeating)
	   		{
	   		MainViewScroll (up, true);
	   		}
	   	else if (event->data.ctlRepeat.controlID == MemoPadMainScrollDownRepeating)
	   		{
	   		MainViewScroll (down, true);
	   		}
	   	
	   	// Repeating controls don't repeat if handled is set true.
			break;


   	case keyDownEvent:		// P13.  Handle the physical scroll buttons
	   	if (event->data.keyDown.chr == pageUpChr)
	   		{
	   		MainViewScroll (up, true);
	   		handled = true;
	   		}
	   	else if (event->data.keyDown.chr == pageDownChr)
	   		{
	   		MainViewScroll (down, true);
	   		handled = true;
	   		}
			break;


  		case tblSelectEvent:  // P12. An entry in the table was selected.
			// Edit the record at the selected table item.
			CurrentRecord = TblGetItemInt (event->data.tblSelect.pTable, 
				event->data.tblSelect.row, event->data.tblSelect.column);
			
			// Go to the edit form.	
			FrmGotoForm(MemoPadEditForm);
			handled = true;
			break;

  		case frmOpenEvent:	// P3. The form was told to open.
  			// P7. Initialize the main form.
			MainViewInit();
			handled = true;
			break;
			
		case menuEvent:		// P2. A menu item was selected.
			// The get info being the only menu item, display the info form.
			
			// First clear the menu status from the display.
			MenuEraseStatus(0);
			
			// Load the info form, then display it.
			frm = FrmInitForm(MemoPadInfoForm);
			FrmDoDialog(frm);
			
			// Delete the info form.
	 		FrmDeleteForm(frm);
			handled = true;
			break;
		}
	return(handled);
}


/***********************************************************************
 *
 * FUNCTION:    ApplicationHandleEvent
 *
 * DESCRIPTION: P3. This routine loads a form resource and sets the event handler for the form.
 *
 * PARAMETERS:  event - a pointer to an EventType structure
 *
 * RETURNED:    True if the event has been handled and should not be
 *						passed to a higher level handler.
 *
 ***********************************************************************/
static Boolean ApplicationHandleEvent(EventPtr event)
{
	FormPtr	frm;
	Int		formId;
	Boolean	handled = false;

	if (event->eType == frmLoadEvent)
		{
		// Load the form resource specified in the event then activate the form.
		formId = event->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);

		// Set the event handler for the form.  The handler of the currently 
		// active form is called by FrmDispatchEvent each time it receives an event.
		switch (formId)
			{
			case MemoPadMainForm:
				FrmSetEventHandler(frm, MainViewHandleEvent);
				break;
		
			case MemoPadEditForm:
				FrmSetEventHandler(frm, EditViewHandleEvent);
				break;
		
			case MemoPadDetailsForm:		// P10.
				FrmSetEventHandler(frm, DetailsFormHandleEvent);
				break;
			}
		handled = true;
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:		EventLoop
 *
 * DESCRIPTION:	A simple loop that obtains events from the Event
 *						Manager and passes them on to various applications and
 *						system event handlers before passing them on to
 *						FrmDispatchEvent for default processing.
 *
 * PARAMETERS:		None.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void EventLoop(void)
{
	EventType	event;
	Word			error;
	
	do
		{
		// Get the next available event.
		EvtGetEvent(&event, evtWaitForever);
		
		// Give the system a chance to handle the event.
		if (! SysHandleEvent(&event))

			// P2. Give the menu bar a chance to update and handle the event.	
			if (! MenuHandleEvent(0, &event, &error))

				// P3. Give the application a chance to handle the event.
				if (! ApplicationHandleEvent(&event))

					// P3. Let the form object provide default handling of the event.
					FrmDispatchEvent(&event);
		}
	while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:		PilotMain
 *
 * DESCRIPTION:	This function is the equivalent of a main() function
 *						under standard C.  It is called by the Emulator to begin
 *						execution of this application.
 *
 * PARAMETERS:		cmd - command specifying how to launch the application.
 *						cmdPBP - parameter block for the command.
 *						launchFlags - flags used to configure the launch.			
 *
 * RETURNED:		Any applicable error codes.
 *
 ***********************************************************************/
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	Word error;		// P4. error starting the app
	
	// P4. This app makes use of PalmOS 2.0 features.  It will crash if
	// run on an earlier version of PalmOS.  Detect and warn if this happens,
	// then exit.
	error = RomVersionCompatible (version20, launchFlags);
	if (error)
		return error;

	// P14.  Check for a normal launch.
	if (cmd == sysAppLaunchCmdNormalLaunch)
		{
		// Initialize the application's global variables and database.
		if (!StartApplication())
			{
			// P3. Start the first form.
			FrmGotoForm(CurrentView);
				
			// Start the event loop.
			EventLoop();
			
			// P5. Clean up before exiting the applcation.
			StopApplication();
			}
		}
		
	// P14. The system find command uses this to search this application.
	else if (cmd == sysAppLaunchCmdFind)
		{
		Search ((FindParamsPtr)cmdPBP);
		}
	
	
	// P14. If the user wants to goto a record listed in the find dialog
	// it sends the following notification
	else if (cmd == sysAppLaunchCmdGoTo)
		{
		// Check if new global variables are set up.  This occurs when the 
		// system intends for the app to start.
		if (launchFlags & sysAppLaunchFlagNewGlobals) 
			{
			// This occurs when the user wants to see a record in this
			// application's database.  The app must be started and 
			// a view to display the data must be opened.
			if (!StartApplication ())
				{
				GoToRecord ((GoToParamsPtr) cmdPBP, true);
				
				EventLoop ();
				StopApplication ();
				}
			}
		else
			{
			// The app was already running.  Open a form to display the record.
			GoToRecord ((GoToParamsPtr) cmdPBP, false);
			}
		}


	// P14. Launch code sent to running app to save all data.
	// This is neccessary before a find for some applications because
	// some apps do not edit in place and the user might search for
	// data not saved.  Or saving a record might reorder the database.
	// Saving before a find avoids finding a record, saving, resorting,
	// and then displaying a record moved to the position of the found
	// record.
	else if (cmd == sysAppLaunchCmdSaveData)
		{
		FrmSaveAllForms ();
		}
	

	// P17. The database has been synchronized.  Handle any changes
	// in the data (resorting data could be performed).
	else if (cmd == sysAppLaunchCmdSyncNotify)
		{
//		SyncNotification ();
		}
	

	// P17. The database has been created but there isn't an app info block.
	// Either the HotSync app or another application which uses this application's
	// data may request that the application info block be created by this app
	// (because it's the creator of the database).
	else if (cmd == sysAppLaunchCmdInitDatabase)
		{
		MemoPadInitAppInfo (((SysAppLaunchCmdInitDatabaseType*)cmdPBP)->dbP);
		}
	

	return 0;
}
