/*********************************************************************
 *
 * Copyright (c) 1998
 * 3Com/Palm Computing Division.  All rights reserved.
 *
 * Portions copyright (c) 1996-1997
 * Greg Hewgill.  All rights reserved.
 *
 * Portions copyright (c) 1995-1997
 * Illume Software, Inc.  All rights reserved.
 *
 * Portions copyright (c) 1995-1997
 * Bernd Schmidt.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *		This product includes software developed by 3Com and its
 *		contributors.
 *
 * 4. Neither 3Com nor the names of its contributors may be used to
 *    endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE 3COM AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 3COM OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 *******************************************************************/

#include "StringData.h"

#include "SysTraps.h"


/*
	# To generate the list of trap names, set the current directory to
	# the directory holding this file. Then:

	set Space '[ t]+'

	set Src "::Includes:Palm:System:SysTraps.h"
	set Dest "{TempFolder}Tempfile `Date -n`"

	duplicate -y -d "{Src}" "{Dest}"
	setfile -a l "{Dest}"
	Target "{Dest}"
	SaveOnClose "{Dest}" -n

	replace  :/typedef enum[ t]+{/ ""
	replace /sysTrapLastTrapNumber/: ""

	find 
	replace /{Space}={Space}sysTrapBase/ ""

	find 
	replace -c  //// ""

	Find 
	replace -c  /sysTrap([a-zA-Z0-9]+)1,/  ""1","

	Find 
	replace -c  /[ t]*n/ ""
*/

// ---------------------------------------------------------------------------
//		 Names of trap functions
// ---------------------------------------------------------------------------

const char*	kTrapNames[] =
{
	"MemInit",
	"MemInitHeapTable",
	"MemStoreInit",
	"MemCardFormat",
	"MemCardInfo",
	"MemStoreInfo",
	"MemStoreSetInfo",
	"MemNumHeaps",
	"MemNumRAMHeaps",
	"MemHeapID",
	"MemHeapPtr",
	"MemHeapFreeBytes",
	"MemHeapSize",
	"MemHeapFlags",
	"MemHeapCompact",
	"MemHeapInit",
	"MemHeapFreeByOwnerID",
	"MemChunkNew",
	"MemChunkFree",
	"MemPtrNew",
	"MemPtrRecoverHandle",
	"MemPtrFlags",
	"MemPtrSize",
	"MemPtrOwner",
	"MemPtrHeapID",
	"MemPtrCardNo",
	"MemPtrToLocalID",
	"MemPtrSetOwner",
	"MemPtrResize",
	"MemPtrResetLock",
	"MemHandleNew",
	"MemHandleLockCount",
	"MemHandleToLocalID",
	"MemHandleLock",
	"MemHandleUnlock",
	"MemLocalIDToGlobal",
	"MemLocalIDKind",
	"MemLocalIDToPtr",
	"MemMove",
	"MemSet",
	"MemStoreSearch",
	"MemPtrDataStorage",
	"MemKernelInit",
	"MemHandleFree",
	"MemHandleFlags",
	"MemHandleSize",
	"MemHandleOwner",
	"MemHandleHeapID",
	"MemHandleDataStorage",
	"MemHandleCardNo",
	"MemHandleSetOwner",
	"MemHandleResize",
	"MemHandleResetLock",
	"MemPtrUnlock",
	"MemLocalIDToLockedPtr",
	"MemSetDebugMode",
	"MemHeapScramble",
	"MemHeapCheck",
	"MemNumCards",
	"MemDebugMode",
	"MemSemaphoreReserve",
	"MemSemaphoreRelease",
	"MemHeapDynamic",
	"MemNVParams",
	"DmInit",
	"DmCreateDatabase",
	"DmDeleteDatabase",
	"DmNumDatabases",
	"DmGetDatabase",
	"DmFindDatabase",
	"DmDatabaseInfo",
	"DmSetDatabaseInfo",
	"DmDatabaseSize",
	"DmOpenDatabase",
	"DmCloseDatabase",
	"DmNextOpenDatabase",
	"DmOpenDatabaseInfo",
	"DmResetRecordStates",
	"DmGetLastErr",
	"DmNumRecords",
	"DmRecordInfo",
	"DmSetRecordInfo",
	"DmAttachRecord",
	"DmDetachRecord",
	"DmMoveRecord",
	"DmNewRecord",
	"DmRemoveRecord",
	"DmDeleteRecord",
	"DmArchiveRecord",
	"DmNewHandle",
	"DmRemoveSecretRecords",
	"DmQueryRecord",
	"DmGetRecord",
	"DmResizeRecord",
	"DmReleaseRecord",
	"DmGetResource",
	"DmGet1Resource",
	"DmReleaseResource",
	"DmResizeResource",
	"DmNextOpenResDatabase",
	"DmFindResourceType",
	"DmFindResource",
	"DmSearchResource",
	"DmNumResources",
	"DmResourceInfo",
	"DmSetResourceInfo",
	"DmAttachResource",
	"DmDetachResource",
	"DmNewResource",
	"DmRemoveResource",
	"DmGetResourceIndex",
	"DmQuickSort",
	"DmQueryNextInCategory",
	"DmNumRecordsInCategory",
	"DmPositionInCategory",
	"DmSeekRecordInCategory",
	"DmMoveCategory",
	"DmOpenDatabaseByTypeCreator",
	"DmWrite",
	"DmStrCopy",
	"DmGetNextDatabaseByTypeCreator",
	"DmWriteCheck",
	"DmMoveOpenDBContext",
	"DmFindRecordByID",
	"DmGetAppInfoID",
	"DmFindSortPositionV10",
	"DmSet",
	"DmCreateDatabaseFromImage",
	"DbgSrcMessage",
	"DbgMessage",
	"DbgGetMessage",
	"DbgCommSettings",
	"ErrDisplayFileLineMsg",
	"ErrSetJump",
	"ErrLongJump",
	"ErrThrow",
	"ErrExceptionList",
	"SysBroadcastActionCode",
	"SysUnimplemented",
	"SysColdBoot",
	"SysReset",
	"SysDoze",
	"SysAppLaunch",
	"SysAppStartup",
	"SysAppExit",
	"SysSetA5",
	"SysSetTrapAddress",
	"SysGetTrapAddress",
	"SysTranslateKernelErr",
	"SysSemaphoreCreate",
	"SysSemaphoreDelete",
	"SysSemaphoreWait",
	"SysSemaphoreSignal",
	"SysTimerCreate",
	"SysTimerWrite",
	"SysTaskCreate",
	"SysTaskDelete",
	"SysTaskTrigger",
	"SysTaskID",
	"SysTaskUserInfoPtr",
	"SysTaskDelay",
	"SysTaskSetTermProc",
	"SysUILaunch",
	"SysNewOwnerID",
	"SysSemaphoreSet",
	"SysDisableInts",
	"SysRestoreStatus",
	"SysUIAppSwitch",
	"SysCurAppInfoPV20",
	"SysHandleEvent",
	"SysInit",
	"SysQSort",
	"SysCurAppDatabase",
	"SysFatalAlert",
	"SysResSemaphoreCreate",
	"SysResSemaphoreDelete",
	"SysResSemaphoreReserve",
	"SysResSemaphoreRelease",
	"SysSleep",
	"SysKeyboardDialogV10",
	"SysAppLauncherDialog",
	"SysSetPerformance",
	"SysBatteryInfoV20",
	"SysLibInstall",
	"SysLibRemove",
	"SysLibTblEntry",
	"SysLibFind",
	"SysBatteryDialog",
	"SysCopyStringResource",
	"SysKernelInfo",
	"SysLaunchConsole",
	"SysTimerDelete",
	"SysSetAutoOffTime",
	"SysFormPointerArrayToStrings",
	"SysRandom",
	"SysTaskSwitching",
	"SysTimerRead",
	"StrCopy",
	"StrCat",
	"StrLen",
	"StrCompare",
	"StrIToA",
	"StrCaselessCompare",
	"StrIToH",
	"StrChr",
	"StrStr",
	"StrAToI",
	"StrToLower",
	"SerReceiveISP",
	"SlkOpen",
	"SlkClose",
	"SlkOpenSocket",
	"SlkCloseSocket",
	"SlkSocketRefNum",
	"SlkSocketSetTimeout",
	"SlkFlushSocket",
	"SlkSetSocketListener",
	"SlkSendPacket",
	"SlkReceivePacket",
	"SlkSysPktDefaultResponse",
	"SlkProcessRPC",
	"ConPutS",
	"ConGetS",
	"FplInit",
	"FplFree",
	"FplFToA",
	"FplAToF",
	"FplBase10Info",
	"FplLongToFloat",
	"FplFloatToLong",
	"FplFloatToULong",
	"FplMul",
	"FplAdd",
	"FplSub",
	"FplDiv",
	"ScrInit",
	"ScrCopyRectangle",
	"ScrDrawChars",
	"ScrLineRoutine",
	"ScrRectangleRoutine",
	"ScrScreenInfo",
	"ScrDrawNotify",
	"ScrSendUpdateArea",
	"ScrCompressScanLine",
	"ScrDeCompressScanLine",
	"TimGetSeconds",
	"TimSetSeconds",
	"TimGetTicks",
	"TimInit",
	"TimSetAlarm",
	"TimGetAlarm",
	"TimHandleInterrupt",
	"TimSecondsToDateTime",
	"TimDateTimeToSeconds",
	"TimAdjust",
	"TimSleep",
	"TimWake",
	"CategoryCreateListV10",
	"CategoryFreeListV10",
	"CategoryFind",
	"CategoryGetName",
	"CategoryEditV10",
	"CategorySelectV10",
	"CategoryGetNext",
	"CategorySetTriggerLabel",
	"CategoryTruncateName",
	"ClipboardAddItem",
	"ClipboardCheckIfItemExist",
	"ClipboardGetItem",
	"CtlDrawControl",
	"CtlEraseControl",
	"CtlHideControl",
	"CtlShowControl",
	"CtlGetValue",
	"CtlSetValue",
	"CtlGetLabel",
	"CtlSetLabel",
	"CtlHandleEvent",
	"CtlHitControl",
	"CtlSetEnabled",
	"CtlSetUsable",
	"CtlEnabled",
	"EvtInitialize",
	"EvtAddEventToQueue",
	"EvtCopyEvent",
	"EvtGetEvent",
	"EvtGetPen",
	"EvtSysInit",
	"EvtGetSysEvent",
	"EvtProcessSoftKeyStroke",
	"EvtGetPenBtnList",
	"EvtSetPenQueuePtr",
	"EvtPenQueueSize",
	"EvtFlushPenQueue",
	"EvtEnqueuePenPoint",
	"EvtDequeuePenStrokeInfo",
	"EvtDequeuePenPoint",
	"EvtFlushNextPenStroke",
	"EvtSetKeyQueuePtr",
	"EvtKeyQueueSize",
	"EvtFlushKeyQueue",
	"EvtEnqueueKey",
	"EvtDequeueKeyEvent",
	"EvtWakeup",
	"EvtResetAutoOffTimer",
	"EvtKeyQueueEmpty",
	"EvtEnableGraffiti",
	"FldCopy",
	"FldCut",
	"FldDrawField",
	"FldEraseField",
	"FldFreeMemory",
	"FldGetBounds",
	"FldGetTextPtr",
	"FldGetSelection",
	"FldHandleEvent",
	"FldPaste",
	"FldRecalculateField",
	"FldSetBounds",
	"FldSetText",
	"FldGetFont",
	"FldSetFont",
	"FldSetSelection",
	"FldGrabFocus",
	"FldReleaseFocus",
	"FldGetInsPtPosition",
	"FldSetInsPtPosition",
	"FldSetScrollPosition",
	"FldGetScrollPosition",
	"FldGetTextHeight",
	"FldGetTextAllocatedSize",
	"FldGetTextLength",
	"FldScrollField",
	"FldScrollable",
	"FldGetVisibleLines",
	"FldGetAttributes",
	"FldSetAttributes",
	"FldSendChangeNotification",
	"FldCalcFieldHeight",
	"FldGetTextHandle",
	"FldCompactText",
	"FldDirty",
	"FldWordWrap",
	"FldSetTextAllocatedSize",
	"FldSetTextHandle",
	"FldSetTextPtr",
	"FldGetMaxChars",
	"FldSetMaxChars",
	"FldSetUsable",
	"FldInsert",
	"FldDelete",
	"FldUndo",
	"FldSetDirty",
	"FldSendHeightChangeNotification",
	"FldMakeFullyVisible",
	"FntGetFont",
	"FntSetFont",
	"FntGetFontPtr",
	"FntBaseLine",
	"FntCharHeight",
	"FntLineHeight",
	"FntAverageCharWidth",
	"FntCharWidth",
	"FntCharsWidth",
	"FntDescenderHeight",
	"FntCharsInWidth",
	"FntLineWidth",
	"FrmInitForm",
	"FrmDeleteForm",
	"FrmDrawForm",
	"FrmEraseForm",
	"FrmGetActiveForm",
	"FrmSetActiveForm",
	"FrmGetActiveFormID",
	"FrmGetUserModifiedState",
	"FrmSetNotUserModified",
	"FrmGetFocus",
	"FrmSetFocus",
	"FrmHandleEvent",
	"FrmGetFormBounds",
	"FrmGetWindowHandle",
	"FrmGetFormId",
	"FrmGetFormPtr",
	"FrmGetNumberOfObjects",
	"FrmGetObjectIndex",
	"FrmGetObjectId",
	"FrmGetObjectType",
	"FrmGetObjectPtr",
	"FrmHideObject",
	"FrmShowObject",
	"FrmGetObjectPosition",
	"FrmSetObjectPosition",
	"FrmGetControlValue",
	"FrmSetControlValue",
	"FrmGetControlGroupSelection",
	"FrmSetControlGroupSelection",
	"FrmCopyLabel",
	"FrmSetLabel",
	"FrmGetLabel",
	"FrmSetCategoryLabel",
	"FrmGetTitle",
	"FrmSetTitle",
	"FrmAlert",
	"FrmDoDialog",
	"FrmCustomAlert",
	"FrmHelp",
	"FrmUpdateScrollers",
	"FrmGetFirstForm",
	"FrmVisible",
	"FrmGetObjectBounds",
	"FrmCopyTitle",
	"FrmGotoForm",
	"FrmPopupForm",
	"FrmUpdateForm",
	"FrmReturnToForm",
	"FrmSetEventHandler",
	"FrmDispatchEvent",
	"FrmCloseAllForms",
	"FrmSaveAllForms",
	"FrmGetGadgetData",
	"FrmSetGadgetData",
	"FrmSetCategoryTrigger", 
	"UIInitialize",
	"UIReset",
	"InsPtInitialize",
	"InsPtSetLocation",
	"InsPtGetLocation",
	"InsPtEnable",
	"InsPtEnabled",
	"InsPtSetHeight",
	"InsPtGetHeight",
	"InsPtCheckBlink",
	"LstSetDrawFunction",
	"LstDrawList",
	"LstEraseList",
	"LstGetSelection",
	"LstGetSelectionText",
	"LstHandleEvent",
	"LstSetHeight",
	"LstSetSelection",
	"LstSetListChoices",
	"LstMakeItemVisible",
	"LstGetNumberOfItems",
	"LstPopupList",
	"LstSetPosition",
	"MenuInit",
	"MenuDispose",
	"MenuHandleEvent",
	"MenuDrawMenu",
	"MenuEraseStatus",
	"MenuGetActiveMenu",
	"MenuSetActiveMenu",
	"RctSetRectangle",
	"RctCopyRectangle",
	"RctInsetRectangle",
	"RctOffsetRectangle",
	"RctPtInRectangle",
	"RctGetIntersection",
	"TblDrawTable",
	"TblEraseTable",
	"TblHandleEvent",
	"TblGetItemBounds",
	"TblSelectItem",
	"TblGetItemInt",
	"TblSetItemInt",
	"TblSetItemStyle",
	"TblUnhighlightSelection",
	"TblSetRowUsable",
	"TblGetNumberOfRows",
	"TblSetCustomDrawProcedure",
	"TblSetRowSelectable",
	"TblRowSelectable",
	"TblSetLoadDataProcedure",
	"TblSetSaveDataProcedure",
	"TblGetBounds",
	"TblSetRowHeight",
	"TblGetColumnWidth",
	"TblGetRowID",
	"TblSetRowID",
	"TblMarkRowInvalid",
	"TblMarkTableInvalid",
	"TblGetSelection",
	"TblInsertRow",
	"TblRemoveRow",
	"TblRowInvalid",
	"TblRedrawTable",
	"TblRowUsable",
	"TblReleaseFocus",
	"TblEditing",
	"TblGetCurrentField",
	"TblSetColumnUsable",
	"TblGetRowHeight",
	"TblSetColumnWidth",
	"TblGrabFocus",
	"TblSetItemPtr",
	"TblFindRowID",
	"TblGetLastUsableRow",
	"TblGetColumnSpacing",
	"TblFindRowData",
	"TblGetRowData",
	"TblSetRowData",
	"TblSetColumnSpacing",
	"WinCreateWindow",
	"WinCreateOffscreenWindow",
	"WinDeleteWindow",
	"WinInitializeWindow",
	"WinAddWindow",
	"WinRemoveWindow",
	"WinSetActiveWindow",
	"WinSetDrawWindow",
	"WinGetDrawWindow",
	"WinGetActiveWindow",
	"WinGetDisplayWindow",
	"WinGetFirstWindow",
	"WinEnableWindow",
	"WinDisableWindow",
	"WinGetWindowFrameRect",
	"WinDrawWindowFrame",
	"WinEraseWindow",
	"WinSaveBits",
	"WinRestoreBits",
	"WinCopyRectangle",
	"WinScrollRectangle",
	"WinGetDisplayExtent",
	"WinGetWindowExtent",
	"WinDisplayToWindowPt",
	"WinWindowToDisplayPt",
	"WinGetClip",
	"WinSetClip",
	"WinResetClip",
	"WinClipRectangle",
	"WinDrawLine",
	"WinDrawGrayLine",
	"WinEraseLine",
	"WinInvertLine",
	"WinFillLine",
	"WinDrawRectangle",
	"WinEraseRectangle",
	"WinInvertRectangle",
	"WinDrawRectangleFrame",
	"WinDrawGrayRectangleFrame",
	"WinEraseRectangleFrame",
	"WinInvertRectangleFrame",
	"WinGetFramesRectangle",
	"WinDrawChars",
	"WinEraseChars",
	"WinInvertChars",
	"WinGetPattern",
	"WinSetPattern",
	"WinSetUnderlineMode",
	"WinDrawBitmap",
	"WinModal",
	"WinGetWindowBounds",
	"WinFillRectangle",
	"WinDrawInvertedChars",
	"PrefOpenPreferenceDBV10",
	"PrefGetPreferences",
	"PrefSetPreferences",
	"PrefGetAppPreferencesV10",
	"PrefSetAppPreferencesV10",
	"SndInit",
	"SndSetDefaultVolume",
	"SndGetDefaultVolume",
	"SndDoCmd",
	"SndPlaySystemSound",
	"AlmInit",
	"AlmCancelAll",
	"AlmAlarmCallback",
	"AlmSetAlarm",
	"AlmGetAlarm",
	"AlmDisplayAlarm",
	"AlmEnableNotification",
	"HwrGetRAMMapping",
	"HwrMemWritable",
	"HwrMemReadable",
	"HwrDoze",
	"HwrSleep",
	"HwrWake",
	"HwrSetSystemClock",
	"HwrSetCPUDutyCycle",
	"HwrLCDInit",
	"HwrLCDSleep",
	"HwrTimerInit",
	"HwrCursor",
	"HwrBatteryLevel",
	"HwrDelay",
	"HwrEnableDataWrites",
	"HwrDisableDataWrites",
	"HwrLCDBaseAddr",
	"HwrLCDDrawBitmap",
	"HwrTimerSleep",
	"HwrTimerWake",
	"HwrLCDWake",
	"HwrIRQ1Handler",
	"HwrIRQ2Handler",
	"HwrIRQ3Handler",
	"HwrIRQ4Handler",
	"HwrIRQ5Handler",
	"HwrIRQ6Handler",
	"HwrDockSignals",
	"HwrPluggedIn",
	"Crc16CalcBlock",
	"SelectDayV10",
	"SelectTime",
	"DayDrawDaySelector",
	"DayHandleEvent",
	"DayDrawDays",
	"DayOfWeek",
	"DaysInMonth",
	"DayOfMonth",
	"DateDaysToDate",
	"DateToDays",
	"DateAdjust",
	"DateSecondsToDate",
	"DateToAscii",
	"DateToDOWDMFormat",
	"TimeToAscii",
	"Find",
	"FindStrInStr",
	"FindSaveMatch",
	"FindGetLineBounds",
	"FindDrawHeader",
	"PenOpen",
	"PenClose",
	"PenGetRawPen",
	"PenCalibrate",
	"PenRawToScreen",
	"PenScreenToRaw",
	"PenResetCalibration",
	"PenSleep",
	"PenWake",
	"ResLoadForm",
	"ResLoadMenu",
	"FtrInit",
	"FtrUnregister",
	"FtrGet",
	"FtrSet",
	"FtrGetByIndex",
	"GrfInit",
	"GrfFree",
	"GrfGetState",
	"GrfSetState",
	"GrfFlushPoints",
	"GrfAddPoint",
	"GrfInitState",
	"GrfCleanState",
	"GrfMatch",
	"GrfGetMacro",
	"GrfFilterPoints",
	"GrfGetNumPoints",
	"GrfGetPoint",
	"GrfFindBranch",
	"GrfMatchGlyph",
	"GrfGetGlyphMapping",
	"GrfGetMacroName",
	"GrfDeleteMacro",
	"GrfAddMacro",
	"GrfGetAndExpandMacro",
	"GrfProcessStroke",
	"GrfFieldChange",
	"GetCharSortValue",
	"GetCharAttr",
	"GetCharCaselessValue",
	"PwdExists",
	"PwdVerify",
	"PwdSet",
	"PwdRemove",
	"GsiInitialize",
	"GsiSetLocation",
	"GsiEnable",
	"GsiEnabled",
	"GsiSetShiftState",
	"KeyInit",
	"KeyHandleInterrupt",
	"KeyCurrentState",
	"KeyResetDoubleTap",
	"KeyRates",
	"KeySleep",
	"KeyWake",
	"DlkControl",				
	"DlkStartServer",
	"DlkGetSyncInfo",
	"DlkSetLogEntry",
	"Unused2",				
	"SysLibLoad",			
	"SndPlaySmf",			
	"SndCreateMidiList",	
	"AbtShowAbout",
	"MdmDial",
	"MdmHangUp",
	"DmSearchRecord",
	"SysInsertionSort",
	"DmInsertionSort",
	"LstSetTopItem",
	"SclSetScrollBar",
	"SclDrawScrollBar",
	"SclHandleEvent",
	"SysMailboxCreate",
	"SysMailboxDelete",
	"SysMailboxFlush",
	"SysMailboxSend",
	"SysMailboxWait",
	"SysTaskWait",
	"SysTaskWake",
	"SysTaskWaitClr",
	"SysTaskSuspend",
	"SysTaskResume",
	"CategoryCreateList",
	"CategoryFreeList",
	"CategoryEditV20",
	"CategorySelect",
	"DmDeleteCategory",
	"SysEvGroupCreate",
	"SysEvGroupSignal",
	"SysEvGroupRead",
	"SysEvGroupWait",
	"EvtEventAvail",
	"EvtSysEventAvail",
	"StrNCopy",
	"KeySetMask",
	"SelectDay",
	"PrefGetPreference",
	"PrefSetPreference",
	"PrefGetAppPreferences",
	"PrefSetAppPreferences",
	"FrmPointInTitle",
	"StrNCat",
	"MemCmp",
	"TblSetColumnEditIndicator",
	"FntWordWrap",
	"FldGetScrollValues",
	"SysCreateDataBaseList",
	"SysCreatePanelList",
	"DlkDispatchRequest",
	"StrPrintF",
	"StrVPrintF",
	"PrefOpenPreferenceDB",
	"SysGraffitiReferenceDialog",
	"SysKeyboardDialog",
	"FntWordWrapReverseNLines",
	"FntGetScrollValues",
	"TblSetRowStaticHeight",
	"TblHasScrollBar",
	"SclGetScrollBar",
	"FldGetNumberOfBlankLines",
	"SysTicksPerSecond",
	"HwrBacklight",
	"DmDatabaseProtect",
	"TblSetBounds",
	"StrNCompare",
	"StrNCaselessCompare",	
	"PhoneNumberLookup",
	"FrmSetMenu",
	"EncDigestMD5",
	"DmFindSortPosition",
	"SysBinarySearch",
	"SysErrString",
	"SysStringByIndex",
	"EvtAddUniqueEventToQueue",
	"StrLocalizeNumber",
	"StrDelocalizeNumber",
	"LocGetNumberSeparators",
	"MenuSetActiveMenuRscID",
	"LstScrollList",
	"CategoryInitialize",
	"EncDigestMD4",
	"EncDES",
	"LstGetVisibleItems",
	"WinSetWindowBounds",
	"CategorySetName",
	"FldSetInsertionPoint",
	"FrmSetObjectBounds",
	"WinSetColors",
	"FlpDispatch",
	"FlpEmDispatch",
	"ExgInit",
	"ExgConnect",
	"ExgPut",
	"ExgGet",
	"ExgAccept",
	"ExgDisconnect",
	"ExgSend",
	"ExgReceive",
	"ExgRegisterData",
	"ExgNotifyReceive",
	"ExgControl",
	"PrgStartDialog",
	"PrgStopDialog",
	"PrgUpdateDialog",
	"PrgHandleEvent",
	"ImcReadFieldNoSemicolon",
	"ImcReadFieldQuotablePrintable",
	"ImcReadPropertyParameter",
	"ImcReadAllPropertyParameters",
	"ImcReadWhiteSpace",
	"ImcWriteQuotedPrintable",
	"ImcWriteNoSemicolon",
	"ImcStringIsAscii",
	"TblGetItemFont",
	"TblSetItemFont",
	"FontSelect",
	"FntDefineFont",
	"CategoryEdit",
	"SysGetOSVersionString",
	"SysBatteryInfo",
	"SysUIBusy",
	"WinValidateHandle",
	"FrmValidatePtr",
	"CtlValidatePointer",
	"WinMoveWindow",
	"FrmAddSpaceForObject",
	"FrmNewForm",
	"CtlNewControl",
	"FldNewField",
	"LstNewList",
	"FrmNewLabel",
	"FrmNewBitmap",
	"FrmNewGadget",
	"FileOpen",
	"FileClose",
	"FileDelete",
	"FileRead",
	"FileWrite",
	"FileSeek",
	"FileTell",
	"FileTruncate",
	"FileControl",
	"FrmActiveState",
	"SysGetAppInfo",
	"SysGetStackInfo",
	"ScrDisplayMode",
	"HwrLCDGetDepth",
	"HwrGetROMToken",
	"DbgControl",
	"ExgDBRead",
	"ExgDBWrite",
	"SysGremlins",
	"FrmRemoveObject",
	"SysReserved1",
	"SysReserved2",
	"SysReserved3",
	"SysReserved4",
	"LastTrapNumber"
};

const int kNumTrapNames = sizeof(kTrapNames) / sizeof (kTrapNames[0]) - 1;

// Make sure we've got them all.  This line will generate
// an error if sysNumTraps (in SysTraps.h) is different from kNumTraps.

static int dummy[ (sysNumTraps == kNumTrapNames) ? 1 : 0 ];


// ---------------------------------------------------------------------------
//		 Names of exception types
// ---------------------------------------------------------------------------

const char*	kExceptionNames[] =
{
	"initStack",							// initial stack pointer
	"initPC",								// initial PC

	"busErr",								// 08 / 2
	"addressErr",							// 0C / 3
	"illegalInstr",							// 10 / 4
	"divideByZero",							// 14 / 5
	"chk",									// 18 / 6
	"trap",									// 1C / 7
	"privilege",							// 20 / 8
	"trace",								// 24 / 9
	"aTrap",								// 28 / A
	"fTrap",								// 2C / B
	"reserved12",							// 30 / C
	"coproc",								// 34 / D
	"formatErr",							// 38 / E
	"unitializedInt",						// 3C / F

	"reserved0",							// 40-5C / 10-17
	"reserved1",
	"reserved2",
	"reserved3",
	"reserved4",
	"reserved5",
	"reserved6",
	"reserved7",

	"spuriousInt",							// 60 / 18
	"autoVec1",								// 64 / 19
	"autoVec2",								// 68 / 1A
	"autoVec3",								// 6C / 1B
	"autoVec4",								// 70 / 1C
	"autoVec5",								// 74 / 1D
	"autoVec6",								// 78 / 1E
	"autoVec7",								// 7C / 1F

	"trap0",								// 80 - BC / 20 - 2F
	"trap1",
	"trap2",
	"trap3",
	"trap4",
	"trap5",
	"trap6",
	"trap7",
	"trap8",
	"trap9",
	"trap10",
	"trap11",
	"trap12",
	"trap13",
	"trap14",
	"trap15",

	"unassigned0",							// C0 - FC / 30 - 3F
	"unassigned1",
	"unassigned2",
	"unassigned3",
	"unassigned4",
	"unassigned5",
	"unassigned6",
	"unassigned7",
	"unassigned8",
	"unassigned9",
	"unassigned10",
	"unassigned11",
	"unassigned12",
	"unassigned13",
	"unassigned14",
	"unassigned15"
};


// ---------------------------------------------------------------------------
//		 Error strings.  In C array format for easy cross-platformability.
// ---------------------------------------------------------------------------

const char*	kErrorStrings[] =
{
	"",						// Dummy "No error" string
	"An error occurred!",	// Generic error string

	"There isn't enough memory to load the ROM file!  Please give the application more memory.",
	"The ROM file is missing!  To download the ROM from your Palm device, choose \"Download ROM...\" from the File menu.",
	"The ROM file could not be loaded!  The file may be damaged, in use, or the wrong size.",
	"The ROM data does not appear to be valid!  Please use this ROM file with caution.",
	"This Debug 2.0 ROM is not supported. Please download the latest from http://www.palm.com.",
	"The ROM file couldn't be saved!  There may not be enough disk space or the file may be in use.",

	"The size of the selected RAM image is invalid!",
	"There isn't enough memory to create the RAM partition!  Please give the application more memory.",
	"The RAM file could not be loaded!  The file may be damaged or already in use.",
	"The RAM file couldn't be saved!  There may not be enough disk space or the file may be in use.",

	"There isn't enough memory to hold data needed by the CPU emulator!  Please give the application more memory.",
	"There isn't enough memory to hold data needed by the CPU emulator!  Please give the application more memory.",

	"The selected file could not be opened!  It may already be opened by another application.",
	"There isn't enough memory to load the selected file!  Please give the application more memory.",
	"Due to an unknown error, the selected could not be loaded!",
	"The selected application is too large to use!",
	"Due to a database error, the selected application could not be loaded!",
	"That application is currently in use!  Please switch to another application before loading.",

	"There isn't enough memory to download the ROM!  Please give the application more memory and try again.",
	"The serial port is in use!  Please shut down HotSync or any other application that may be using the serial port.",
	"Could not find the ROM header in the received data. Lower the baud rate and try again.",
	"An unexpected error occurred while transferring the ROM!  The serial connection may have been broken.",

	"The menu bar could not be loaded!  The application may be damaged or out of memory.",
	"The main window could not be loaded!  The application may be damaged or out of memory.",
	"The case picture could not be loaded!  The application may be damaged or out of memory.",
	"The application could not be initialized!  The application may be damaged or out of memory.",
	"An unknown exception occurred!",
	"There isnt enough memory to resize the display!  Please give the application more memory.",

	""
};
