/*********************************************************************
 *
 * Copyright (c) 1998
 * 3Com/Palm Computing Division.  All rights reserved.
 *
 * Portions copyright (c) 1996-1997
 * Greg Hewgill.  All rights reserved.
 *
 * Portions copyright (c) 1995-1997
 * Illume Software, Inc.  All rights reserved.
 *
 * Portions copyright (c) 1995-1997
 * Bernd Schmidt.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *		This product includes software developed by 3Com and its
 *		contributors.
 *
 * 4. Neither 3Com nor the names of its contributors may be used to
 *    endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE 3COM AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 3COM OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 *******************************************************************/

#include <windows.h>
#pragma hdrstop

#include "Registry.h"

const char*	kDefaultRegistryKey	= "Software\\3Com\\Palm OS Emulator";


// ---------------------------------------------------------------------------
//		 SetRegistry
// ---------------------------------------------------------------------------

void SetRegistry(const char* key,
				 const char* name,
				 DWORD type,
				 const void* data,
				 DWORD datasize)
{
	HKEY	k;
	DWORD	d;

	if (RegCreateKeyEx (HKEY_CURRENT_USER, key, 0, NULL, 0, KEY_WRITE, NULL, &k, &d) == ERROR_SUCCESS)
	{
		RegSetValueEx (k, name, 0, type, (BYTE *)data, datasize);
		RegCloseKey (k);
	}
}


// ---------------------------------------------------------------------------
//		 GetRegistry
// ---------------------------------------------------------------------------

BOOL GetRegistry(const char* key,
				 const char* name,
				 DWORD type,
				 void* data,
				 DWORD datasize)
{
	BOOL	ok = FALSE;
	HKEY	k;

	if (RegOpenKeyEx (HKEY_CURRENT_USER, key, 0, KEY_READ, &k) == ERROR_SUCCESS)
	{
		DWORD	t;
		if (RegQueryValueEx (k, (char*) name, NULL, &t, (BYTE*) data, &datasize) == ERROR_SUCCESS)
		{
			ok = t == type;
		}

		RegCloseKey (k);
	}

	return ok;
}


// ---------------------------------------------------------------------------
//		 SetRegistryString
// ---------------------------------------------------------------------------

void SetRegistryString(const char* name,
					   const char* data,
					   const char* key)
{
	SetRegistry (key, name, REG_SZ, data, strlen (data) + 1);
}


// ---------------------------------------------------------------------------
//		 GetRegistryString
// ---------------------------------------------------------------------------

BOOL GetRegistryString(const char* name,
					   char* data,
					   DWORD datasize,
					   const char* key)
{
	return GetRegistry (key, name, REG_SZ, data, datasize);
}


// ---------------------------------------------------------------------------
//		 SetRegistryDword
// ---------------------------------------------------------------------------

void SetRegistryDword(const char* name,
					  DWORD data,
					  const char* key)
{
	SetRegistry (key, name, REG_DWORD, &data, sizeof (data));
}


// ---------------------------------------------------------------------------
//		 GetRegistryDword
// ---------------------------------------------------------------------------

BOOL GetRegistryDword(const char* name,
					  DWORD& data,
					  const char* key)
{
	return GetRegistry (key, name, REG_DWORD, &data, sizeof (data));
}


// ---------------------------------------------------------------------------
//		 SetRegistryBool
// ---------------------------------------------------------------------------

void SetRegistryBool(const char* name,
					 BOOL data,
					 const char* key)
{
	SetRegistry (key, name, REG_DWORD, &data, sizeof (data));
}


// ---------------------------------------------------------------------------
//		 GetRegistryBool
// ---------------------------------------------------------------------------

BOOL GetRegistryBool(const char* name,
					 BOOL& data,
					 const char* key)
{
	return GetRegistry (key, name, REG_DWORD, &data, sizeof (data));
}


// ---------------------------------------------------------------------------
//		 SetRegistryBinary
// ---------------------------------------------------------------------------

void SetRegistryBinary(const char* name,
					   const void* data,
					   DWORD datasize,
					   const char* key)
{
	SetRegistry (key, name, REG_BINARY, data, datasize);
}


// ---------------------------------------------------------------------------
//		 GetRegistryBinary
// ---------------------------------------------------------------------------

BOOL GetRegistryBinary(const char* name,
					   void* data,
					   DWORD datasize,
					   const char* key)
{
	return GetRegistry (key, name, REG_BINARY, data, datasize);
}
