/*********************************************************************
 *
 * Copyright (c) 1998
 * 3Com/Palm Computing Division.  All rights reserved.
 *
 * Portions copyright (c) 1996-1997
 * Greg Hewgill.  All rights reserved.
 *
 * Portions copyright (c) 1995-1997
 * Illume Software, Inc.  All rights reserved.
 *
 * Portions copyright (c) 1995-1997
 * Bernd Schmidt.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *		This product includes software developed by 3Com and its
 *		contributors.
 *
 * 4. Neither 3Com nor the names of its contributors may be used to
 *    endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE 3COM AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 3COM OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 *******************************************************************/

#include <windows.h>
#pragma hdrstop

#include "Settings.h"
#include "Registry.h"

TSettings Settings;

TSettings::TSettings()
{
	Window.x = CW_USEDEFAULT;
	Window.y = CW_USEDEFAULT;
	Scale = 1;
	Realistic = TRUE;
	Backlight = FALSE;

	OSVERSIONINFO osv;	
	osv.dwOSVersionInfoSize = sizeof(osv);
	GetVersionEx(&osv);

	Sounds = osv.dwPlatformId == VER_PLATFORM_WIN32_NT;
	FPS = 20;
	RamSize = 1024;
	strcpy(CommPort, "None");
	DebugAtStartup = FALSE;
	PrcPath[0] = 0;

	for (int i = 0; i < MRU_COUNT; i++)
	{
		PrcMru[i][0] = 0;
	}

	strcpy(RomFile, "palm.rom");

	DWORD x;

	if (GetRegistryDword("WindowX", x))
	{
		Window.x = x;
	}

	if (GetRegistryDword("WindowY", x))
	{
		Window.y = x;
	}

	if (GetRegistryDword("Scale", x))
	{
		if (x < 1)
		{
			x = 0;
		}
		else if (x > 2)
		{
			x = 2;
		}
		Scale = x;
	}

	BOOL b;

	if (GetRegistryBool("Realistic", b))
	{
		Realistic = b;
	}

	if (GetRegistryBool("Backlight", b))
	{
		Backlight = b;
	}

	if (GetRegistryBool("Sounds", b))
	{
		Sounds = b;
	}

	if (GetRegistryDword("FPS", x) && x > 0)
	{
		FPS = x;
	}

	if (GetRegistryDword("RamSize", x) && x > 0)
	{
		RamSize = x;
	}

	char s[MAX_PATH];

	if (GetRegistryString("CommPort", s, sizeof(s)))
	{
		strcpy(CommPort, s);
	}

	if (GetRegistryBool("DebugAtStartup", b))
	{
		DebugAtStartup = b;
	}

	if (GetRegistryString("PrcPath", s, sizeof(s)))
	{
		strcpy(PrcPath, s);
	}

	for (i = 0; i < MRU_COUNT; i++)
	{
		char buf[20];
		wsprintf(buf, "PrcMru.%d", i);
		if (GetRegistryString(buf, s, sizeof(s)))
		{
			strcpy(PrcMru[i], s);
		}
	}

	if (GetRegistryString("RomFile", s, sizeof(s)))
	{
		strcpy(RomFile, s);
	}
}



void TSettings::Save(HWND hwnd)
{
	RECT wr;
	GetWindowRect(hwnd, &wr);

	SetRegistryDword("WindowX", wr.left);
	SetRegistryDword("WindowY", wr.top);
	SetRegistryDword("Scale", Scale);
	SetRegistryDword("Realistic", Realistic);
	SetRegistryDword("Backlight", Backlight);
	SetRegistryDword("Sounds", Sounds);
	SetRegistryDword("FPS", FPS);
	SetRegistryDword("RamSize", RamSize);
	SetRegistryString("CommPort", CommPort);
	SetRegistryDword("DebugAtStartup", DebugAtStartup);
	SetRegistryString("PrcPath", PrcPath);

	for (int i = 0; i < MRU_COUNT; i++)
	{
		if (PrcMru[i][0])
		{
			char buf[20];
			wsprintf(buf, "PrcMru.%d", i);
			SetRegistryString(buf, PrcMru[i]);
		}
	}

	SetRegistryString("RomFile", RomFile);
}


void TSettings::UpdateMru(const char* name)
{
	for (int i = 0; i < MRU_COUNT - 1; i++)
	{
		if (stricmp(name, PrcMru[i]) == 0)
		{
			break;
		}
	}

	if (i > 0)
	{
		MoveMemory(PrcMru[1], PrcMru[0], i * MAX_PATH);
	}

	strcpy(PrcMru[0], name);
}
